@@ -10,7 +10,7 @@ from qualys_etl.etld_lib import etld_lib_extract_transform_load as etld_lib_extr
 def asset_inventory_extract(asset_last_updated, last_seen_assetid, batch_number_str,
                             qualys_headers_dict, cred_dict, file_info_dict) -> dict:
 
-    start_msg_asset_inventory_extract()
+    begin_asset_inventory_04_extract(message=f"start batch: {batch_number_str}")
     bearer = cred_dict['bearer']
     page_size = 300
 
@@ -32,15 +32,17 @@ def asset_inventory_extract(asset_last_updated, last_seen_assetid, batch_number_
         output_file=json_file,
         cred_dict=cred_dict,
         qualys_headers_multiprocessing_dict=qualys_headers_dict,
-        multi_proc_batch_number=batch_number_str,
+        batch_number_formatted=batch_number_str,
         extract_validation_type='json',
         compression_method=etld_lib_config.asset_inventory_open_file_compression_method)
-    end_msg_asset_inventory_extract()
+
+    end_asset_inventory_04_extract(message=f"start batch: {batch_number_str}")
     return cred_dict  # For 401 Edge Case
 
 
 def asset_inventory_extract_count(asset_last_updated, last_seen_assetid, batch_number_str, qualys_headers_dict, cred_dict, file_info_dict):
 
+    begin_asset_inventory_04_extract(message=f"start extract count for asset_last_updated: {asset_last_updated}")
     bearer = cred_dict['bearer']
     url = f"https://{cred_dict['gateway_fqdn_server']}/rest/2.0/count/am/asset"
     url = f"{url}?assetLastUpdated={asset_last_updated}&lastSeenAssetId={last_seen_assetid}"
@@ -59,10 +61,11 @@ def asset_inventory_extract_count(asset_last_updated, last_seen_assetid, batch_n
         output_file=json_file,
         cred_dict=cred_dict,
         qualys_headers_multiprocessing_dict=qualys_headers_dict,
-        multi_proc_batch_number=batch_number_str,
+        batch_number_formatted=batch_number_str,
         extract_validation_type='json')
 
     asset_inventory_log_count(json_file=json_file)
+    end_asset_inventory_04_extract(message=f"start extract count for asset_last_updated: {asset_last_updated}")
 
 
 def asset_inventory_log_count(json_file):
@@ -82,12 +85,12 @@ def asset_inventory_log_count(json_file):
         exit(1)
 
 
-def start_msg_asset_inventory_extract():
-    etld_lib_functions.logger.info(f"start")
+def begin_asset_inventory_04_extract(message=""):
+    etld_lib_functions.logger.info(f"start {message}")
 
 
-def end_msg_asset_inventory_extract():
-    etld_lib_functions.logger.info(f"end")
+def end_asset_inventory_04_extract(message=""):
+    etld_lib_functions.logger.info(f"end   {message}")
 
 
 def main(args=None):
@@ -95,7 +98,7 @@ def main(args=None):
 
 
 if __name__ == "__main__":
-    etld_lib_functions.main(my_logger_prog_name='asset_inventory_extract')
+    etld_lib_functions.main(my_logger_prog_name='asset_inventory_04_extract_from_qualys')
     etld_lib_config.main()
     etld_lib_credentials.main()
     main()
