@@ -10,6 +10,7 @@ from qualys_etl.etld_lib import etld_lib_functions
 from qualys_etl.etld_lib import etld_lib_datetime
 from qualys_etl.etld_lib import etld_lib_extract_transform_load
 from qualys_etl.etld_lib import etld_lib_sqlite_tables
+from qualys_etl.etld_lib import etld_lib_csv_distribution
 
 
 def transform_epoch_dates(item):
@@ -57,38 +58,62 @@ def insert_one_row_into_table_q_asset_inventory(
         sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
         table_name: str,
         table_fields: list,
+        table_field_types: list,
         asset_item_dict: dict,
         counter_obj,
-        counter_obj_duplicates):
+        counter_obj_duplicates,
+        batch_date: str,
+        batch_number: str,
+        batch_name: str,
+        csv_distribution_dict=None
+):
 
-    def prepare_asset_inventory_field(field_data, field_name_tmp):
-        if field_data is None:
-            field_data = ""
-        elif 'Date' in field_name_tmp:
-            field_data = field_data.replace("T", " ").replace("Z", "")
-            field_data = re.sub("\\..*$", "", field_data)
-        elif 'lastBoot' in field_name_tmp:
-            field_data = field_data.replace("T", " ").replace("Z", "")
-            field_data = re.sub("\\..*$", "", field_data)
-        elif isinstance(field_data, int):
-            field_data = str(field_data)
-        elif not isinstance(field_data, str):
-            field_data = json.dumps(field_data)
+    # def prepare_asset_inventory_field(field_data, field_name_tmp):
+    #     if field_data is None:
+    #         field_data = ""
+    #     elif 'Date' in field_name_tmp:
+    #         field_data = field_data.replace("T", " ").replace("Z", "")
+    #         field_data = re.sub("\\..*$", "", field_data)
+    #     elif 'lastBoot' in field_name_tmp:
+    #         field_data = field_data.replace("T", " ").replace("Z", "")
+    #         field_data = re.sub("\\..*$", "", field_data)
+    #     elif isinstance(field_data, int):
+    #         field_data = str(field_data)
+    #     elif not isinstance(field_data, str):
+    #         field_data = json.dumps(field_data)
+    #
+    #     return field_data
+    #
+    # row_in_sqlite_form = []
+    # for field_name in table_fields:  # Iterate through expected columns (contract)
+    #     if field_name in asset_item_dict.keys():  # Iterate through columns found in dictionary
+    #         asset_item_dict[field_name] = \
+    #             sqlite_obj.prepare_database_field_asset_inventory(asset_item_dict[field_name],
+    #                                                               field_name,
+    #                                                               csv_column_types=table_field_types)
+    #         row_in_sqlite_form.append(asset_item_dict[field_name])
+    #     else:
+    #         row_in_sqlite_form.append("")  # Ensure blank is added to each required empty field
 
-        return field_data
-
-    row_in_sqlite_form = []
-    for field_name in table_fields:  # Iterate through expected columns (contract)
-        if field_name in asset_item_dict.keys():  # Iterate through columns found in dictionary
-            asset_item_dict[field_name] = \
-                prepare_asset_inventory_field(asset_item_dict[field_name], field_name)
-            row_in_sqlite_form.append(asset_item_dict[field_name])
-        else:
-            row_in_sqlite_form.append("")  # Ensure blank is added to each required empty field
+    row_in_sqlite_form = sqlite_obj.prepare_database_row_asset_inventory(
+        item_dict=asset_item_dict,
+        csv_columns=table_fields,
+        csv_column_types=table_field_types,
+        batch_name=batch_name,
+        batch_date=batch_date,
+        batch_number=batch_number)
 
     result = sqlite_obj.insert_unique_row_ignore_duplicates(table_name, row_in_sqlite_form)
     if result is True:
         counter_obj.display_counter_to_log()
+        # WRITE CSV ROW
+        if etld_lib_config.asset_inventory_distribution_csv_flag and csv_distribution_dict is not None:
+            #
+            # Write one row to Q_Asset_Inventory CSV File
+            #
+            qids_csv_obj: etld_lib_csv_distribution.CsvDistribution = \
+                csv_distribution_dict['q_asset_inventory_csv_obj']
+            qids_csv_obj.write_csv_row_to_file_handle(row_in_sqlite_form)
     else:
         counter_obj_duplicates.display_counter_to_log()
 
@@ -96,7 +121,11 @@ def insert_one_row_into_table_q_asset_inventory(
 def insert_one_row_into_table_q_asset_inventory_software_assetid(
         asset_item_dict: dict,
         sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
-        counter_obj):
+        counter_obj,
+        batch_date: str,
+        batch_number: str,
+        batch_name: str
+):
 
     if asset_item_dict['softwareListData'] is not None:
         for software_item in asset_item_dict['softwareListData']['software']:
@@ -116,6 +145,7 @@ def insert_one_row_into_table_q_asset_inventory_software_assetid(
                     sqlite_row)
                 if result is True:
                     counter_obj.display_counter_to_log()
+                    # WRITE CSV ROW
                 else:
                     pass
                     #etld_lib_functions.logger.info("found dups in software assetid")
@@ -124,7 +154,11 @@ def insert_one_row_into_table_q_asset_inventory_software_assetid(
 def insert_one_row_into_table_q_asset_inventory_software_os_unique(
         asset_item_dict: dict,
         sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
-        counter_obj):
+        counter_obj,
+        batch_date: str,
+        batch_number: str,
+        batch_name: str,
+):
 
     if asset_item_dict['softwareListData'] is not None:
         for software_item in asset_item_dict['softwareListData']['software']:
@@ -167,6 +201,7 @@ def insert_one_row_into_table_q_asset_inventory_software_os_unique(
                     sqlite_row)
                 if result is True:
                     counter_obj.display_counter_to_log()
+                    # WRITE CSV ROW
 
 
 def drop_and_create_all_tables(sqlite_obj):
@@ -185,6 +220,7 @@ def drop_and_create_all_tables(sqlite_obj):
     sqlite_obj.drop_and_recreate_table(
         table_name=etld_lib_config.asset_inventory_table_name_software_os_unique,
         csv_columns=etld_lib_config.asset_inventory_software_os_unique_csv_columns(),
+        csv_column_types=etld_lib_config.asset_inventory_software_os_unique_csv_column_types(),
         key=['fullName', 'osName'])
 
     sqlite_obj.drop_and_recreate_table(
@@ -196,7 +232,7 @@ def drop_and_create_all_tables(sqlite_obj):
 
 def create_counter_objects(database_type='sqlite'):
     counter_obj_asset_inventory = etld_lib_functions.DisplayCounterToLog(
-        display_counter_at=10000,
+        display_counter_at=25000,
         logger_func=etld_lib_functions.logger.info,
         display_counter_log_message=f"rows added to {database_type} "
                                     f"table {etld_lib_config.asset_inventory_table_name}")
@@ -206,12 +242,12 @@ def create_counter_objects(database_type='sqlite'):
         display_counter_log_message=f"duplicate rows bypassed, not written to {database_type} "
                                     f"table {etld_lib_config.asset_inventory_table_name}")
     counter_obj_software_os = etld_lib_functions.DisplayCounterToLog(
-        display_counter_at=10000,
+        display_counter_at=25000,
         logger_func=etld_lib_functions.logger.info,
         display_counter_log_message=f"rows added to {database_type} "
                                     f"table {etld_lib_config.asset_inventory_table_name_software_os_unique}")
     counter_obj_software_assetid = etld_lib_functions.DisplayCounterToLog(
-        display_counter_at=10000,
+        display_counter_at=25000,
         logger_func=etld_lib_functions.logger.info,
         display_counter_log_message=f"rows added to {database_type} "
                                     f"table {etld_lib_config.asset_inventory_table_name_software_assetid}")
@@ -223,27 +259,42 @@ def create_counter_objects(database_type='sqlite'):
     return counter_objects
 
 
-def insert_one_asset_into_multiple_tables(asset_item, sqlite_obj, counter_objects):
+def insert_one_asset_into_multiple_tables(asset_item, sqlite_obj, counter_objects, csv_distribution_dict=None,
+                                          batch_date="", batch_number="", batch_name=""):
     item = transform_epoch_dates(asset_item)
     insert_one_row_into_table_q_asset_inventory_software_os_unique(
         asset_item_dict=item,
         sqlite_obj=sqlite_obj,
-        counter_obj=counter_objects['counter_obj_software_os'])
+        counter_obj=counter_objects['counter_obj_software_os'],
+        batch_date=batch_date,
+        batch_number=batch_number,
+        batch_name=batch_name
+    )
     insert_one_row_into_table_q_asset_inventory_software_assetid(
         asset_item_dict=item,
         sqlite_obj=sqlite_obj,
-        counter_obj=counter_objects['counter_obj_software_assetid'])
+        counter_obj=counter_objects['counter_obj_software_assetid'],
+        batch_date=batch_date,
+        batch_number=batch_number,
+        batch_name=batch_name
+    )
     insert_one_row_into_table_q_asset_inventory(
         sqlite_obj=sqlite_obj,
         table_name=etld_lib_config.asset_inventory_table_name,
         table_fields=etld_lib_config.asset_inventory_csv_columns(),
+        table_field_types=etld_lib_config.asset_inventory_csv_column_types(),
         asset_item_dict=item,
         counter_obj=counter_objects['counter_obj_asset_inventory'],
-        counter_obj_duplicates=counter_objects['counter_obj_asset_inventory_duplicates']
+        counter_obj_duplicates=counter_objects['counter_obj_asset_inventory_duplicates'],
+        csv_distribution_dict=csv_distribution_dict,
+        batch_date=batch_date,
+        batch_number=batch_number,
+        batch_name=batch_name
     )
 
 
 def transform_and_load_queue_of_json_files_into_sqlite(queue_of_json_files,):
+    begin_asset_inventory_05_transform_load(message="queue_of_json_files_through_multiprocesssing")
     file_path = "EXCEPTION"
     try:
         sqlite_asset_inventory = etld_lib_sqlite_tables.SqliteObj(etld_lib_config.asset_inventory_sqlite_file)
@@ -270,42 +321,77 @@ def transform_and_load_queue_of_json_files_into_sqlite(queue_of_json_files,):
                 batch_date = etld_lib_extract_transform_load.get_batch_date_from_filename(file_name=file_path)
                 etld_lib_functions.logger.info(f"Received batch file from multiprocessing Queue: {batch_name}")
 
-                status_name, status_detail_dict, status_count = \
-                    sqlite_asset_inventory.update_status_table(
+                sqlite_asset_inventory.update_status_table(
                         batch_date=batch_date, batch_number=batch_number,
                         total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                         status_table_name=etld_lib_config.asset_inventory_status_table_name,
                         status_table_columns=etld_lib_config.status_table_csv_columns(),
                         status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                        status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='begin')
+                        status_name_column=etld_lib_config.asset_inventory_table_name, status_column='begin')
 
                 all_items = json.load(read_file)
-                for item in all_items['assetListData']['asset']:
-                    insert_one_asset_into_multiple_tables(item, sqlite_asset_inventory, counter_objects)
+                csv_distribution_dict = {}
+                if etld_lib_config.host_list_detection_distribution_csv_flag:
+                    q_asset_inventory_csv_info =  etld_lib_csv_distribution.get_csv_info_dict(
+                        batch_date=batch_date,
+                        batch_number=batch_number,
+                        table_name=etld_lib_config.asset_inventory_table_name,
+                        csv_file_open_method=etld_lib_config.asset_inventory_open_file_compression_method,
+                        distribution_dir=etld_lib_config.asset_inventory_distribution_dir
+                    )
+                    with q_asset_inventory_csv_info['compression_method'](
+                            q_asset_inventory_csv_info['target_tmp_csv_file_path'], 'wt', newline='') as \
+                            q_asset_inventory_target_tmp_csv_file_handle:
+                        q_asset_inventory_csv_info['csv_obj'].set_csv_writer_with_file_handle(
+                            q_asset_inventory_target_tmp_csv_file_handle)
+                        csv_distribution_dict['q_asset_inventory_csv_obj'] = q_asset_inventory_csv_info['csv_obj']
+                        for item in all_items['assetListData']['asset']:
+                            insert_one_asset_into_multiple_tables(
+                                item, sqlite_asset_inventory, counter_objects,
+                                csv_distribution_dict=csv_distribution_dict,
+                                batch_date=batch_date, batch_number=batch_number, batch_name=batch_name
+                            )
+                    Path(q_asset_inventory_csv_info['target_tmp_csv_file_path']).\
+                            rename(q_asset_inventory_csv_info['target_csv_file_path'])
+                else:
+                    for item in all_items['assetListData']['asset']:
+                        insert_one_asset_into_multiple_tables(
+                            item, sqlite_asset_inventory, counter_objects, csv_distribution_dict=None,
+                            batch_date=batch_date, batch_number=batch_number, batch_name=batch_name
+                        )
+
                 sqlite_asset_inventory.commit_changes()
 
-                status_name, status_detail_dict, status_count = \
-                    sqlite_asset_inventory.update_status_table(
+                sqlite_asset_inventory.update_status_table(
                         batch_date=batch_date, batch_number=batch_number,
                         total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                         status_table_name=etld_lib_config.asset_inventory_status_table_name,
                         status_table_columns=etld_lib_config.status_table_csv_columns(),
                         status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                        status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='end')
+                        status_name_column=etld_lib_config.asset_inventory_table_name, status_column='end')
 
                 etld_lib_functions.logger.info(
                     f"Committed batch file from multiprocessing Queue into Database: {batch_name}")
 
         etld_lib_functions.logger.info(f"Completed processing Queue of files")
 
-        status_name, status_detail_dict, status_count = \
-            sqlite_asset_inventory.update_status_table(
+        sqlite_asset_inventory.update_status_table(
                 batch_date=batch_date, batch_number=batch_number,
                 total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                 status_table_name=etld_lib_config.asset_inventory_status_table_name,
                 status_table_columns=etld_lib_config.status_table_csv_columns(),
                 status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='final')
+                status_name_column=etld_lib_config.asset_inventory_table_name, status_column='final')
+
+        sqlite_asset_inventory.update_status_table(
+            batch_date=batch_date,
+            batch_number=0,
+            total_rows_added_to_database=0,
+            status_table_name=etld_lib_config.asset_inventory_status_table_name,
+            status_table_columns=etld_lib_config.status_table_csv_columns(),
+            status_table_column_types=etld_lib_config.status_table_csv_column_types(),
+            status_name_column='ALL_TABLES_LOADED_SUCCESSFULLY',
+            status_column='final')
 
         sqlite_asset_inventory.commit_changes()
         sqlite_asset_inventory.close_connection()
@@ -313,25 +399,25 @@ def transform_and_load_queue_of_json_files_into_sqlite(queue_of_json_files,):
         counter_objects['counter_obj_asset_inventory'].display_final_counter_to_log()
         counter_objects['counter_obj_software_os'].display_final_counter_to_log()
         counter_objects['counter_obj_software_assetid'].display_final_counter_to_log()
-        end_msg_asset_inventory_to_sqlite()
 
     except Exception as e:
         etld_lib_functions.logger.error(f"Exception: {e}")
         etld_lib_functions.logger.error(f"Potential JSON File corruption detected: {file_path}")
         exit(1)
+    end_asset_inventory_05_transform_load(message="queue_of_json_files_through_multiprocesssing")
 
 
-def end_msg_asset_inventory_to_sqlite():
-    etld_lib_functions.logger.info(f"end")
+def begin_asset_inventory_05_transform_load(message=""):
+    etld_lib_functions.logger.info(f"start {message}")
 
 
-def start_msg_asset_inventory_sqlite():
-    etld_lib_functions.logger.info("start")
+def end_asset_inventory_05_transform_load(message=""):
+    etld_lib_functions.logger.info(f"end  {message}")
 
 
 def transform_and_load_all_json_files_into_sqlite():
+    begin_asset_inventory_05_transform_load(message="directory_listing_of_json_files")
     json_file = "EXCEPTION"
-    start_msg_asset_inventory_sqlite()
     try:
         sqlite_asset_inventory = etld_lib_sqlite_tables.SqliteObj(etld_lib_config.asset_inventory_sqlite_file)
         drop_and_create_all_tables(sqlite_asset_inventory)
@@ -348,39 +434,55 @@ def transform_and_load_all_json_files_into_sqlite():
                 batch_date = etld_lib_extract_transform_load.get_batch_date_from_filename(file_name=json_file)
                 etld_lib_functions.logger.info(f"Received batch file from directory: {batch_name}")
 
-                status_name, status_detail_dict, status_count = \
-                    sqlite_asset_inventory.update_status_table(
+                sqlite_asset_inventory.update_status_table(
                         batch_date=batch_date, batch_number=batch_number,
                         total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                         status_table_name=etld_lib_config.asset_inventory_status_table_name,
                         status_table_columns=etld_lib_config.status_table_csv_columns(),
                         status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                        status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='begin')
+                        status_name_column=etld_lib_config.asset_inventory_table_name, status_column='begin')
 
                 all_items = json.load(read_file)
+                # Open CSV File
                 for item in all_items['assetListData']['asset']:
-                    insert_one_asset_into_multiple_tables(item, sqlite_asset_inventory, counter_objects)
+                    insert_one_asset_into_multiple_tables(
+                        item,
+                        sqlite_asset_inventory,
+                        counter_objects,
+                        csv_distribution_dict=None,
+                        batch_date=batch_date,
+                        batch_number=batch_number,
+                        batch_name=batch_name
+                    )
 
                 etld_lib_functions.logger.info(
                     f"Added batch file from directory into Database: {batch_name}")
 
-                status_name, status_detail_dict, status_count = \
-                    sqlite_asset_inventory.update_status_table(
+                sqlite_asset_inventory.update_status_table(
                         batch_date=batch_date, batch_number=batch_number,
                         total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                         status_table_name=etld_lib_config.asset_inventory_status_table_name,
                         status_table_columns=etld_lib_config.status_table_csv_columns(),
                         status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                        status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='end')
+                        status_name_column=etld_lib_config.asset_inventory_table_name, status_column='end')
 
-        status_name, status_detail_dict, status_count = \
-            sqlite_asset_inventory.update_status_table(
+        sqlite_asset_inventory.update_status_table(
                 batch_date=batch_date, batch_number=batch_number,
                 total_rows_added_to_database=counter_objects['counter_obj_asset_inventory'].get_counter(),
                 status_table_name=etld_lib_config.asset_inventory_status_table_name,
                 status_table_columns=etld_lib_config.status_table_csv_columns(),
                 status_table_column_types=etld_lib_config.status_table_csv_column_types(),
-                status_name_column='ASSET_INVENTORY_LOAD_STATUS', status_column='final')
+                status_name_column=etld_lib_config.asset_inventory_table_name, status_column='final')
+
+        sqlite_asset_inventory.update_status_table(
+            batch_date=batch_date,
+            batch_number=0,
+            total_rows_added_to_database=0,
+            status_table_name=etld_lib_config.asset_inventory_status_table_name,
+            status_table_columns=etld_lib_config.status_table_csv_columns(),
+            status_table_column_types=etld_lib_config.status_table_csv_column_types(),
+            status_name_column='ALL_TABLES_LOADED_SUCCESSFULLY',
+            status_column='final')
 
         etld_lib_functions.logger.info(f"Completed processing directory of files")
         sqlite_asset_inventory.commit_changes()
@@ -394,7 +496,7 @@ def transform_and_load_all_json_files_into_sqlite():
         etld_lib_functions.logger.error(f"JSON File Issue: {json_file}")
         etld_lib_functions.logger.error(f"Exception: {e}")
         exit(1)
-    end_msg_asset_inventory_to_sqlite()
+    end_asset_inventory_05_transform_load(message="directory_listing_of_json_files")
 
 
 def spawn_multiprocessing_queue_to_transform_and_load_json_files_into_sqlite():
@@ -414,7 +516,7 @@ def main():
 
 
 if __name__ == "__main__":
-    etld_lib_functions.main(my_logger_prog_name='asset_inventory_json_to_sqlite')
+    etld_lib_functions.main(my_logger_prog_name='asset_inventory_05_transform_load_json_to_sqlite')
     etld_lib_config.main()
     etld_lib_config.asset_inventory_json_to_sqlite_via_multiprocessing = False
     main()
