@@ -7,7 +7,7 @@ import yaml
 import resource
 from qualys_etl.etld_lib import etld_lib_functions
 from qualys_etl.etld_lib import etld_lib_datetime
-from qualys_etl.etld_lib import etld_lib_sqlite_tables
+
 
 global setup_completed_flag           # Set to true when completed
 global system_usage_counters          # Dict of system usage counters
@@ -40,6 +40,10 @@ global kb_last_modified_after
 global kb_payload_option
 global kb_extract_dir
 global kb_extract_dir_file_search_blob
+global kb_distribution_dir
+global kb_distribution_dir_file_search_blob
+global kb_distribution_csv_flag
+global kb_distribution_csv_max_field_size
 global kb_xml_file
 global kb_shelve_file
 global kb_sqlite_file
@@ -71,6 +75,10 @@ global host_list_vm_processed_after
 global host_list_payload_option
 global host_list_extract_dir
 global host_list_extract_dir_file_search_blob
+global host_list_distribution_dir
+global host_list_distribution_dir_file_search_blob
+global host_list_distribution_csv_flag
+global host_list_distribution_csv_max_field_size
 # TODO eliminate these variables
 global host_list_xml_file_list
 global host_list_other_xml_file
@@ -107,6 +115,10 @@ global host_list_detection_vm_processed_after
 global host_list_detection_payload_option
 global host_list_detection_extract_dir
 global host_list_detection_extract_dir_file_search_blob
+global host_list_detection_distribution_dir
+global host_list_detection_distribution_dir_file_search_blob
+global host_list_detection_distribution_csv_flag
+global host_list_detection_distribution_csv_max_field_size
 global host_list_detection_xml_file
 global host_list_detection_shelve_file
 global host_list_detection_sqlite_file
@@ -142,6 +154,10 @@ global asset_inventory_json_batch_file
 global asset_inventory_extract_dir
 global asset_inventory_extract_dir_file_search_blob
 global asset_inventory_extract_dir_file_search_blob_two
+global asset_inventory_distribution_dir
+global asset_inventory_distribution_dir_file_search_blob
+global asset_inventory_distribution_csv_flag
+global asset_inventory_distribution_csv_max_field_size
 global asset_inventory_shelve_software_assetid_file
 global asset_inventory_shelve_software_unique_file
 global asset_inventory_shelve_software_os_unique_file
@@ -180,6 +196,7 @@ global asset_inventory_last_seen_asset_id_for_restart
 global test_system_log_file
 global test_system_log_rotate_file
 global test_system_lock_file
+global test_system_test_intermediary_extract_flag
 
 # WAS system global variables
 global was_data_dir
@@ -189,9 +206,17 @@ global was_payload_option
 global was_extract_dir
 global was_extract_dir_file_search_blob
 global was_extract_dir_file_search_blob_webapp
+global was_extract_dir_file_search_blob_webapp_detail
 global was_extract_dir_file_search_blob_finding
+global was_extract_dir_file_search_blob_finding_detail
+global was_extract_dir_file_search_blob_catalog
 global was_extract_dir_file_search_blob_webapp_count
 global was_extract_dir_file_search_blob_finding_count
+global was_extract_dir_file_search_blob_catalog_count
+global was_distribution_dir
+global was_distribution_dir_file_search_blob
+global was_distribution_csv_flag
+global was_distribution_csv_max_field_size
 global was_sqlite_file
 global was_log_file
 global was_lock_file
@@ -200,10 +225,10 @@ global was_concurrency_limit
 global was_multi_proc_batch_size
 global was_limit_hosts
 global was_status_table_name
-global was_application_table_name
-global was_application_detail_table_name
+global was_webapp_table_name
 global was_finding_table_name
-global was_finding_detail_table_name
+global was_catalog_table_name
+global was_q_knowledgebase_in_was_finding_table
 global was_data_files
 global was_json_to_sqlite_via_multiprocessing
 global was_chunk_size_calc
@@ -218,6 +243,124 @@ global was_test_number_of_files_to_extract
 qetl_create_user_dirs_ok_flag = False      # Automatically create unknown user directories default - False
 qetl_manage_user_selected_datetime = None  # Initialize qetl_manage_user datetime to None
 system_usage_counters = []                 # Initialize system usage counters dictionary
+default_distribution_csv_max_field_size = 1000000
+test_system_do_not_test_intermediary_extracts_flag = False  # Set to True when running etld_lib_test_system
+
+# Useful to run thorugh all known workflows.
+etl_workflow_list = ['knowledgebase_etl_workflow','host_list_etl_workflow',
+                     'host_list_detection_etl_workflow', 'asset_inventory_etl_workflow',
+                     'was_etl_workflow', 'test_system_etl_workflow']
+
+
+def get_etl_workflow_data_location_dict(etl_workflow) -> dict:
+
+    # Useful when qetl_manage_user -e validate_etl_[workflow] is executed.
+    etl_test_workflow_no_logs = False
+    if etl_workflow == 'validate_etl_knowledgebase':
+        etl_workflow = 'knowledgebase_etl_workflow'
+    if etl_workflow.startswith('knowledgebase_0'):
+        etl_workflow = 'knowledgebase_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow == 'validate_etl_host_list':
+        etl_workflow = 'host_list_etl_workflow'
+    elif etl_workflow.startswith('host_list_0'):
+        etl_workflow = 'host_list_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow.startswith('host_list_detection_0'):
+        etl_workflow = 'host_list_detection_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow == 'validate_etl_host_list_detection':
+        etl_workflow = 'host_list_detection_etl_workflow'
+    elif etl_workflow.startswith('asset_inventory_0'):
+        etl_workflow = 'asset_inventory_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow == 'validate_etl_asset_inventory':
+        etl_workflow = 'asset_inventory_etl_workflow'
+    elif etl_workflow.startswith('was_0'):
+        etl_workflow = 'was_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow == 'validate_etl_was':
+        etl_workflow = 'was_etl_workflow'
+    elif etl_workflow.startswith('test_system_0'):
+        etl_workflow = 'test_system_etl_workflow'
+        etl_test_workflow_no_logs = True
+    elif etl_workflow == 'validate_etl_test_system':
+        etl_workflow = 'test_system_etl_workflow'
+
+    etl_workflow_location_dict = \
+        {
+            'test_system_etl_workflow':
+                {
+                    'etl_workflow': 'test_system_etl_workflow',
+                    'sqlite_file': None,
+                    'log_file': test_system_log_file,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                },
+            'knowledgebase_etl_workflow':
+                {
+                    'etl_workflow': 'knowledgebase_etl_workflow',
+                    'sqlite_file': kb_sqlite_file,
+                    'log_file': kb_log_file,
+                    'extract_dir': kb_extract_dir,
+                    'distribution_dir': kb_distribution_dir,
+                    'csv_max_field_size': kb_distribution_csv_max_field_size,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                    'csv_distribution_display_counter_at': 25000,
+                    'open_file_compression_method': kb_open_file_compression_method
+                  },
+            'host_list_etl_workflow':
+                {
+                    'etl_workflow': 'host_list_etl_workflow',
+                    'sqlite_file': host_list_sqlite_file,
+                    'log_file': host_list_log_file,
+                    'extract_dir': host_list_extract_dir,
+                    'distribution_dir': host_list_distribution_dir,
+                    'csv_max_field_size': host_list_distribution_csv_max_field_size,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                    'csv_distribution_display_counter_at': 25000,
+                    'open_file_compression_method': host_list_open_file_compression_method
+                },
+            'host_list_detection_etl_workflow':
+                {
+                    'etl_workflow': 'host_list_detection_etl_workflow',
+                    'sqlite_file': host_list_detection_sqlite_file,
+                    'log_file': host_list_detection_log_file,
+                    'extract_dir': host_list_detection_extract_dir,
+                    'distribution_dir': host_list_detection_distribution_dir,
+                    'csv_max_field_size': host_list_detection_distribution_csv_max_field_size,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                    'csv_distribution_display_counter_at': 250000,
+                    'open_file_compression_method': host_list_detection_open_file_compression_method
+                },
+            'asset_inventory_etl_workflow':
+                {
+                    'etl_workflow': 'asset_inventory_etl_workflow',
+                    'sqlite_file': asset_inventory_sqlite_file,
+                    'log_file': asset_inventory_log_file,
+                    'extract_dir': asset_inventory_extract_dir,
+                    'distribution_dir': asset_inventory_distribution_dir,
+                    'csv_max_field_size': asset_inventory_distribution_csv_max_field_size,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                    'csv_distribution_display_counter_at': 10000,
+                    'open_file_compression_method': asset_inventory_open_file_compression_method
+                },
+            'was_etl_workflow':
+                {
+                    'etl_workflow': 'was_etl_workflow',
+                    'sqlite_file': was_sqlite_file,
+                    'log_file': was_log_file,
+                    'extract_dir': was_extract_dir,
+                    'distribution_dir': was_distribution_dir,
+                    'csv_max_field_size': was_distribution_csv_max_field_size,
+                    'etl_test_workflow_no_logs': etl_test_workflow_no_logs,
+                    'csv_distribution_display_counter_at': 10000,
+                    'open_file_compression_method': was_open_file_compression_method
+                }
+         }
+    if etl_workflow in etl_workflow_location_dict:
+        return etl_workflow_location_dict[etl_workflow]
+    else:
+        return {}
 
 
 def prepare_extract_batch_file_name(next_batch_number_str='batch_000001',
@@ -583,6 +726,11 @@ def get_kb_user_config():
     global kb_payload_option
     global kb_extract_dir
     global kb_extract_dir_file_search_blob
+    global kb_distribution_dir
+    global kb_distribution_dir_file_search_blob
+    global kb_distribution_csv_flag
+    global kb_distribution_csv_max_field_size
+    global default_distribution_csv_max_field_size
     global qetl_user_config_settings_yaml_file
     global kb_csv_truncate_cell_limit
     global kb_present_csv_cell_as_json
@@ -601,7 +749,7 @@ def get_kb_user_config():
                                            f"-d option - {kb_last_modified_after}")
         elif kb_last_modified_after == 'default':
             # kb_last_modified_after = etld_lib_datetime.get_utc_date_minus_days(7)
-            # Keep default so knowledgebase_03_extract can set the date to max_date found in database.
+            # Keep default so knowledgebase_04_extract can set the date to max_date found in database.
             pass
         else:
             etld_lib_functions.logger.info(f"kb_last_modified_after yaml - {kb_last_modified_after}")
@@ -619,6 +767,12 @@ def get_kb_user_config():
     kb_extract_dir = Path(kb_data_dir, "knowledgebase_extract_dir")
     kb_extract_dir_file_search_blob = 'kb_utc*'
 
+    kb_distribution_dir = Path(kb_data_dir, "knowledgebase_distribution_dir")
+    kb_distribution_dir_file_search_blob = 'Q_*'
+    kb_distribution_csv_flag = get_attribute_from_config_settings('kb_distribution_csv_flag', False)
+    kb_distribution_csv_max_field_size = \
+        get_attribute_from_config_settings('kb_distribution_csv_max_field_size', default_distribution_csv_max_field_size)
+
     kb_export_dir = get_attribute_from_config_settings('kb_export_dir', 'default')
     set_kb_last_modified_after()
     kb_csv_truncate_cell_limit = 0  # get_attribute_from_config_settings('kb_csv_truncate_cell_limit', 0)
@@ -680,6 +834,9 @@ def setup_kb_vars():
 def get_host_list_user_config():
     global host_list_data_dir
     global host_list_export_dir
+    global host_list_distribution_csv_flag
+    global host_list_distribution_csv_max_field_size
+    global default_distribution_csv_max_field_size
     global qetl_user_config_settings_yaml_file
     global host_list_vm_processed_after
     global host_list_payload_option
@@ -747,6 +904,9 @@ def get_host_list_user_config():
 
     host_list_data_dir = qetl_user_data_dir
     host_list_export_dir = get_attribute_from_config_settings('host_list_export_dir', 'default')
+    host_list_distribution_csv_flag = get_attribute_from_config_settings('host_list_distribution_csv_flag', False)
+    host_list_distribution_csv_max_field_size = \
+        get_attribute_from_config_settings('host_list_distribution_csv_max_field_size', default_distribution_csv_max_field_size)
     host_list_xml_to_sqlite_via_multiprocessing = \
         get_attribute_from_config_settings('host_list_xml_to_sqlite_via_multiprocessing', True)
     host_list_csv_truncate_cell_limit = 0
@@ -772,6 +932,8 @@ def setup_host_list_vars():
     global host_list_data_dir
     global host_list_extract_dir
     global host_list_extract_dir_file_search_blob
+    global host_list_distribution_dir
+    global host_list_distribution_dir_file_search_blob
     # TODO eliminate these variables
     global host_list_xml_file_list
     global host_list_other_xml_file
@@ -793,6 +955,8 @@ def setup_host_list_vars():
     host_list_data_dir = qetl_user_data_dir
     host_list_extract_dir = Path(host_list_data_dir, "host_list_extract_dir")
     host_list_extract_dir_file_search_blob = 'host_list_utc*'
+    host_list_distribution_dir = Path(host_list_data_dir, "host_list_distribution_dir")
+    host_list_distribution_dir_file_search_blob = 'Q_*'
     host_list_other_xml_file = Path(host_list_data_dir, "host_list_other_file.xml")
     host_list_ec2_xml_file = Path(host_list_data_dir, "host_list_ec2_file.xml")
     host_list_gcp_xml_file = Path(host_list_data_dir, "host_list_gcp_file.xml")
@@ -816,6 +980,9 @@ def setup_host_list_vars():
 def get_host_list_detection_user_config():
     global host_list_detection_data_dir
     global host_list_detection_export_dir
+    global host_list_detection_distribution_csv_flag
+    global host_list_detection_distribution_csv_max_field_size
+    global default_distribution_csv_max_field_size
     global host_list_detection_vm_processed_after
     global host_list_detection_payload_option
     global host_list_detection_concurrency_limit
@@ -926,6 +1093,9 @@ def get_host_list_detection_user_config():
     host_list_detection_data_dir = qetl_user_data_dir
     host_list_detection_export_dir = \
         get_attribute_from_config_settings('host_list_detection_export_dir', 'default')
+    host_list_detection_distribution_csv_flag = get_attribute_from_config_settings('host_list_detection_distribution_csv_flag', False)
+    host_list_detection_distribution_csv_max_field_size = \
+        get_attribute_from_config_settings('host_list_detection_distribution_csv_max_field_size', default_distribution_csv_max_field_size)
     host_list_detection_csv_truncate_cell_limit = '0'
     # get_attribute_from_config_settings('host_list_detection_csv_truncate_cell_limit', '0')
 
@@ -964,6 +1134,8 @@ def setup_host_list_detection_vars():
     global host_list_detection_xml_file
     global host_list_detection_extract_dir
     global host_list_detection_extract_dir_file_search_blob
+    global host_list_detection_distribution_dir
+    global host_list_detection_distribution_dir_file_search_blob
     global host_list_detection_shelve_file
     global host_list_detection_sqlite_file
     global host_list_detection_csv_file
@@ -982,6 +1154,8 @@ def setup_host_list_detection_vars():
     host_list_detection_data_dir = Path(qetl_user_data_dir)
     host_list_detection_extract_dir = Path(host_list_detection_data_dir, "host_list_detection_extract_dir")
     host_list_detection_extract_dir_file_search_blob = "host_list_detection_utc*"
+    host_list_detection_distribution_dir = Path(host_list_detection_data_dir, "host_list_detection_distribution_dir")
+    host_list_detection_distribution_dir_file_search_blob = "Q_*"
     host_list_detection_shelve_file = Path(host_list_detection_data_dir, "host_list_detection_shelve")
     host_list_detection_sqlite_file = Path(host_list_detection_data_dir, "host_list_detection_sqlite.db")
     host_list_detection_csv_file = Path(host_list_detection_data_dir, "host_list_detection.csv")
@@ -1001,6 +1175,9 @@ def setup_host_list_detection_vars():
 def get_asset_inventory_user_config():
     global asset_inventory_data_dir
     global asset_inventory_export_dir
+    global asset_inventory_distribution_csv_flag
+    global asset_inventory_distribution_csv_max_field_size
+    global default_distribution_csv_max_field_size
     global asset_inventory_asset_last_updated
     global asset_inventory_payload_option
     global asset_inventory_concurrency_limit
@@ -1034,6 +1211,9 @@ def get_asset_inventory_user_config():
     asset_inventory_data_dir = qetl_user_data_dir
     asset_inventory_export_dir = \
         get_attribute_from_config_settings('asset_inventory_export_dir', 'default')
+    asset_inventory_distribution_csv_flag = get_attribute_from_config_settings('asset_inventory_distribution_csv_flag', False)
+    asset_inventory_distribution_csv_max_field_size = \
+        get_attribute_from_config_settings('asset_inventory_distribution_csv_max_field_size', default_distribution_csv_max_field_size)
     asset_inventory_csv_truncate_cell_limit = '0'
     # get_attribute_from_config_settings('asset_inventory_csv_truncate_cell_limit', '0')
     asset_inventory_asset_last_updated = \
@@ -1080,6 +1260,8 @@ def setup_asset_inventory_vars():
     global asset_inventory_extract_dir
     global asset_inventory_extract_dir_file_search_blob
     global asset_inventory_extract_dir_file_search_blob_two
+    global asset_inventory_distribution_dir
+    global asset_inventory_distribution_dir_file_search_blob
     global asset_inventory_shelve_file
     global asset_inventory_shelve_software_assetid_file
     global asset_inventory_shelve_software_unique_file
@@ -1106,6 +1288,8 @@ def setup_asset_inventory_vars():
     asset_inventory_extract_dir = Path(asset_inventory_data_dir, "asset_inventory_extract_dir")
     asset_inventory_extract_dir_file_search_blob = "asset_inventory_utc*"
     asset_inventory_extract_dir_file_search_blob_two = "asset_inventory_count_utc*"
+    asset_inventory_distribution_dir = Path(asset_inventory_data_dir, "asset_inventory_distribution_dir")
+    asset_inventory_distribution_dir_file_search_blob = "Q_*"
     asset_inventory_temp_shelve_file = Path(asset_inventory_extract_dir, "asset_inventory_temp_shelve.db")
     asset_inventory_shelve_file = Path(asset_inventory_data_dir, "asset_inventory_shelve")
     asset_inventory_shelve_software_assetid_file = \
@@ -1142,6 +1326,9 @@ def setup_asset_inventory_vars():
 def get_was_user_config():
     global was_data_dir
     global was_export_dir
+    global was_distribution_csv_flag
+    global was_distribution_csv_max_field_size
+    global default_distribution_csv_max_field_size
     global was_webapp_last_scan_date
     global was_payload_option
     global was_concurrency_limit
@@ -1157,14 +1344,18 @@ def get_was_user_config():
     global was_test_system_flag
     global was_test_number_of_files_to_extract
 
-    def set_was_application_last_scan_date():
+    def set_was_webapp_last_scan_date():
         global was_webapp_last_scan_date
         if qetl_manage_user_selected_datetime is not None:
             was_webapp_last_scan_date = qetl_manage_user_selected_datetime
-            etld_lib_functions.logger.info(f"was_webapp_last_scan_date set by qetl_manage_user -d option")
+            if was_webapp_last_scan_date.startswith("20"):
+                etld_lib_functions.logger.info(f"was_webapp_last_scan_date set by qetl_manage_user -d option")
+            else:
+                was_webapp_last_scan_date = "2000-01-01T00:00:00Z"
+                etld_lib_functions.logger.info(f"was_webapp_last_scan_date reset to 2000-01-01T00:00:00Z.  reset from qetl_manage_user -d option")
         elif was_webapp_last_scan_date == 'default':
-            was_webapp_last_scan_date = etld_lib_datetime.get_utc_date_minus_days(7)
-            etld_lib_functions.logger.info(f"was_webapp_last_scan_date default set to utc.now minus 7 days")
+            was_webapp_last_scan_date = etld_lib_datetime.get_utc_date_minus_days(365)
+            etld_lib_functions.logger.info(f"was_webapp_last_scan_date default set to utc.now minus 365 days")
 
         etld_lib_functions.logger.info(f"was_webapp_last_scan_date - "
                                        f"{was_webapp_last_scan_date}")
@@ -1172,6 +1363,9 @@ def get_was_user_config():
     was_data_dir = qetl_user_data_dir
     was_export_dir = \
         get_attribute_from_config_settings('was_export_dir', 'default')
+    was_distribution_csv_flag = get_attribute_from_config_settings('was_distribution_csv_flag', False)
+    was_distribution_csv_max_field_size = \
+        get_attribute_from_config_settings('was_distribution_csv_max_field_size', default_distribution_csv_max_field_size)
     was_webapp_last_scan_date = \
         get_attribute_from_config_settings('was_webapp_last_scan_date', 'default')
     was_payload_option = get_attribute_from_config_settings('was_payload_option', '')
@@ -1192,7 +1386,7 @@ def get_was_user_config():
     was_test_system_flag = get_attribute_from_config_settings('was_test_system_flag', False)
     was_test_number_of_files_to_extract = \
         get_attribute_from_config_settings('was_test_number_of_files_to_extract', 3)
-    set_was_application_last_scan_date()
+    set_was_webapp_last_scan_date()
 
     etld_lib_functions.logger.info(f"was_export_dir - {was_export_dir}")
     etld_lib_functions.logger.info(f"was_extract_dir - {was_extract_dir}")
@@ -1204,35 +1398,48 @@ def setup_was_vars():
     global was_extract_dir
     global was_extract_dir_file_search_blob
     global was_extract_dir_file_search_blob_webapp
+    global was_extract_dir_file_search_blob_webapp_detail
     global was_extract_dir_file_search_blob_finding
+    global was_extract_dir_file_search_blob_finding_detail
+    global was_extract_dir_file_search_blob_catalog
     global was_extract_dir_file_search_blob_webapp_count
     global was_extract_dir_file_search_blob_finding_count
+    global was_extract_dir_file_search_blob_catalog_count
+    global was_distribution_dir
+    global was_distribution_dir_file_search_blob
     global was_sqlite_file
     global was_log_file
     global was_lock_file
     global was_log_rotate_file
     global was_status_table_name
-    global was_application_table_name
-    global was_application_detail_table_name
+    global was_webapp_table_name
+    global was_catalog_table_name
     global was_finding_table_name
-    global was_finding_detail_table_name
+    global was_q_knowledgebase_in_was_finding_table
     global was_data_files
 
     was_data_dir = Path(qetl_user_data_dir)
     was_extract_dir = Path(was_data_dir, "was_extract_dir")
     was_extract_dir_file_search_blob = "was_*_utc_*"
     was_extract_dir_file_search_blob_webapp = "was_webapp_utc_*"
+    was_extract_dir_file_search_blob_webapp_detail = "was_webapp_detail_utc_*"
     was_extract_dir_file_search_blob_finding = "was_finding_utc_*"
+    was_extract_dir_file_search_blob_finding_detail = "was_finding_detail_utc_*"
+    was_extract_dir_file_search_blob_catalog = "was_catalog_utc_*"
     was_extract_dir_file_search_blob_webapp_count = "was_count_webapp_utc_*"
     was_extract_dir_file_search_blob_finding_count = "was_count_finding_utc_*"
+    was_extract_dir_file_search_blob_catalog_count = "was_count_catalog_utc_*"
+    was_distribution_dir = Path(was_data_dir, "was_distribution_dir")
+    was_distribution_dir_file_search_blob = "Q_*"
     was_sqlite_file = Path(was_data_dir, "was_sqlite.db")
     was_log_file = Path(qetl_user_log_dir, "was.log")
     was_lock_file = Path(qetl_user_log_dir, ".was.lock")
     was_log_rotate_file = Path(qetl_user_log_dir, "was.1.log")
+    was_q_knowledgebase_in_was_finding_table = 'Q_KnowledgeBase_In_Q_WAS_FINDING'
+    was_webapp_table_name = 'Q_WAS_WebApp'
+    was_catalog_table_name = 'Q_WAS_Catalog'
     was_finding_table_name = 'Q_WAS_Finding'
-    was_finding_detail_table_name = 'Q_WAS_Finding_Detail'
-    was_application_table_name = 'Q_WAS_Application'
-    was_application_detail_table_name = 'Q_WAS_Application_Detail'
+
     was_status_table_name = 'Q_WAS_Status'
     was_data_files = [was_sqlite_file]
 
@@ -1271,8 +1478,9 @@ def setup_requests_module_tls_verify_status():
 
 
 def run_log_csv_columns():
-    csv_columns = ['LOG_DATETIME', 'LOG_LEVEL', 'LOG_WORKFLOW_FUNCTION_NAME', 'LOG_WORKFLOW_DATETIME', 'LOG_USERNAME',
-                   'LOG_FUNCTION_NAME', 'LOG_MESSAGE']
+    csv_columns = ['LOG_DATETIME', 'LOG_LEVEL', 'LOG_WORKFLOW', 'LOG_USERNAME',
+                   'LOG_FUNCTION', 'LOG_MESSAGE']
+    return csv_columns
 
 
 def run_log_csv_column_types():
@@ -1287,7 +1495,7 @@ def kb_csv_columns():
                    'SUPPORTED_MODULES', 'IS_DISABLED',
                    'CVE_LIST', 'THREAT_INTELLIGENCE', 'CORRELATION', 'BUGTRAQ_LIST', 'SOFTWARE_LIST',
                    'VENDOR_REFERENCE_LIST', 'CVSS', 'CVSS_V3', 'CHANGE_LOG_LIST', 'DISCOVERY', 'PCI_REASONS',
-                   'BATCH_DATE', 'BATCH_NUMBER'
+                   'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
                    ]
 
     return csv_columns
@@ -1309,7 +1517,7 @@ def host_list_csv_columns():  # Return list of csv columns
         'CLOUD_PROVIDER_TAGS', 'LAST_VULN_SCAN_DATETIME', 'LAST_VM_SCANNED_DATE', 'LAST_VM_SCANNED_DURATION',
         'LAST_VM_AUTH_SCANNED_DATE', 'LAST_VM_AUTH_SCANNED_DURATION', 'LAST_COMPLIANCE_SCAN_DATETIME', 'OWNER',
         'COMMENTS', 'USER_DEF', 'ASSET_GROUP_IDS', 'ASSET_RISK_SCORE', 'ASSET_CRITICALITY_SCORE', 'ARS_FACTORS',
-        'BATCH_DATE', 'BATCH_NUMBER'
+        'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
     ]
 
     return csv_columns
@@ -1318,7 +1526,7 @@ def host_list_csv_columns():  # Return list of csv columns
 def host_list_csv_column_types():
 
     csv_columns = {
-        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER', 'BATCH_NUMBER': 'INTEGER'
+        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER'
     }
     return csv_columns
 
@@ -1346,14 +1554,14 @@ def host_list_detection_host_csv_columns():  # Return list of csv columns
                    'NETBIOS', 'QG_HOSTID',
                    'LAST_SCAN_DATETIME', 'LAST_VM_SCANNED_DATE', 'LAST_VM_SCANNED_DURATION',
                    'LAST_VM_AUTH_SCANNED_DATE', 'LAST_VM_AUTH_SCANNED_DURATION', 'LAST_PC_SCANNED_DATE',
-                   'BATCH_DATE', 'BATCH_NUMBER'
+                   'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
                    ]
     return csv_columns
 
 
 def host_list_detection_host_csv_column_types():  # Return list of csv columns
     csv_columns = {
-        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER', 'BATCH_NUMBER': 'INTEGER'
+        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER'
     }
     return csv_columns
 
@@ -1373,15 +1581,15 @@ def host_list_detection_qids_csv_columns():  # Return list of csv columns
                    'LAST_TEST_DATETIME', 'LAST_UPDATE_DATETIME', 'LAST_FIXED_DATETIME', 'FIRST_REOPENED_DATETIME',
                    'LAST_REOPENED_DATETIME', 'TIMES_REOPENED', 'SERVICE', 'IS_IGNORED', 'IS_DISABLED',
                    'AFFECT_RUNNING_KERNEL', 'AFFECT_RUNNING_SERVICE', 'AFFECT_EXPLOITABLE_CONFIG',
-                   'QDS', 'QDS_FACTORS',
-                   'RESULTS', 'BATCH_DATE', 'BATCH_NUMBER'
+                   'QDS', 'QDS_FACTORS', 'RESULTS',
+                   'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
                   ]
     return csv_columns
 
 
 def host_list_detection_qids_csv_column_types():  # Return list of csv columns
     csv_columns = {
-        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER', 'QID': 'INTEGER', 'BATCH_NUMBER': 'INTEGER',
+        'ID': 'INTEGER', 'ASSET_ID': 'INTEGER', 'QID': 'INTEGER',
         'TIMES_REOPENED': 'INTEGER', 'TIMES_FOUND': 'INTEGER'
     }
     return csv_columns
@@ -1400,7 +1608,7 @@ def asset_inventory_csv_columns():  # Return list of csv columns updated 0.6.111
         'inventory', 'activity', 'tagList', 'serviceList', 'lastLocation', 'criticality',
         'businessInformation', 'assignedLocation', 'businessAppListData',
         'riskScore', 'passiveSensor', 'domain', 'subdomain', 'whois', 'isp', 'asn',
-        'processor', 'TRUNCATED_FIELD_LIST'
+        'processor', 'missingSoftware', 'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
                    ]
     return csv_columns
 
@@ -1435,6 +1643,11 @@ def asset_inventory_software_unique_csv_columns():
     ]
     return csv_columns
 
+def asset_inventory_software_unique_csv_column_types():
+
+    csv_columns = {
+    }
+    return csv_columns
 
 def asset_inventory_software_os_unique_csv_columns():
     csv_columns = [
@@ -1444,11 +1657,139 @@ def asset_inventory_software_os_unique_csv_columns():
     ]
     return csv_columns
 
+def asset_inventory_software_os_unique_csv_column_types():
+
+    csv_columns = {
+    }
+    return csv_columns
+
+
+def was_webapp_csv_columns():
+    csv_columns = [
+        'id', 'name', 'url', 'os', 'owner', 'scope', 'subDomain', 'domains', 'uris', 'attributes', 'defaultProfile',
+        'defaultScanner', 'defaultScannerTags', 'scannerLocked', 'progressiveScanning', 'redundancyLinks',
+        'maxRedundancyLinks', 'urlBlacklist', 'urlWhitelist', 'postDataBlacklist', 'logoutRegexList', 'authRecords',
+        'dnsOverrides', 'useRobots', 'useSitemap', 'headers', 'malwareMonitoring', 'malwareNotification',
+        'malwareTaskId', 'malwareScheduling', 'tags', 'comments', 'isScheduled', 'lastScan', 'createdBy',
+        'createdDate', 'updatedBy', 'updatedDate', 'screenshot', 'proxy', 'config', 'crawlingScripts',
+        'lastScanStatus', 'removeFromSubscription', 'reactivateIfExists', 'postmanCollection', 'swaggerFile',
+        'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
+    ]
+    return csv_columns
+
+
+def was_webapp_csv_column_types():
+
+    csv_columns = {
+        'id': 'INTEGER'
+    }
+    return csv_columns
+
+
+def was_finding_csv_columns():
+    # "Finding": {
+    #     "webApp": {
+    #         "id": 78440423,
+    #         "tags": {
+    #             "list": [
+    #                 {
+    #                     "Tag": {
+    #                         "name": "Web Application Assets",
+    #                         "id": 7472465
+    #                     }
+    #                 }
+    #             ],
+    #             "count": 1
+    #         },
+    #         "url": "http://owaspbwa/bWAPP/login.php",
+    #         "name": "bWAPP"
+    #     },
+    #     "id": 149802,
+    #     "url": "http://owaspbwa/bWAPP/login.php",
+    #     "lastDetectedDate": "2018-07-03T20:06:11Z",
+    #     "qid": 0,
+    #     "etl_workflow_validation_type": "VULNERABILITY",
+    #     "status": "NEW",
+    #     "timesDetected": 1,
+    #     "potential": "false",
+    #     "findingType": "BURP",
+    #     "lastTestedDate": "2018-07-03T20:06:11Z",
+    #     "firstDetectedDate": "2018-07-03T20:06:11Z",
+    #     "isIgnored": "false",
+    #     "severity": "5",
+    #     "uniqueId": "e5028940-ffe9-4a6d-90c8-68724b96ed89"
+    # }
+
+    # csv_columns = [
+    #     'id', 'webApp_id', 'webApp_name', 'webApp_tags', 'webApp_url',
+    #     'url', 'lastDetectedDate', 'qid', 'etl_workflow_validation_type', 'status', 'timesDetected',
+    #     'potential', 'findingType', 'lastTestedDate', 'firstDetectedDate', 'isIgnored',
+    #     'severity', 'uniqueId'
+    # ]
+    # Note: group_0 and function_0 contain "group" and "function" data for DB Compatability.
+    csv_columns = [
+        'id', 'webApp_id', 'webApp_name', 'webApp_tags', 'webApp_url', 'uniqueId',
+        'qid', 'name', 'potential', 'findingType', 'group_0',
+        'cwe', 'owasp', 'wasc', 'param', 'function_0', 'content', 'resultList', 'severity',
+        'originalSeverity', 'url', 'status', 'firstDetectedDate', 'lastDetectedDate',
+        'timesDetected', 'webApp', 'patch', 'isIgnored', 'ignoredReason', 'ignoredBy',
+        'ignoredDate', 'ignoredComment', 'reactivateIn', 'reactivateDate', 'externalRef',
+        'severityComment', 'editedSeverityUser', 'editedSeverityDate', 'retest',
+        'sslData', 'cvssV3', 'history', 'updatedDate',
+        'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
+    ]
+    return csv_columns
+
+
+def was_finding_csv_column_types():
+
+    csv_columns = {
+        'webApp_id': 'INTEGER', 'id': 'INTEGER', 'severity': 'INTEGER', 'timesDetected': 'INTEGER', 'qid': 'INTEGER'
+    }
+    return csv_columns
+
+
+def was_catalog_csv_columns():
+    # {
+    #     "Catalog": {
+    #         "fqdn": "ks355837.kimsufi.com",
+    #         "updatedDate": "2022-01-18T19:45:09Z",
+    #         "id": 11403514,
+    #         "source": "VM_SCAN",
+    #         "operatingSystem": "Ubuntu / Linux 2.6.x",
+    #         "port": "443",
+    #         "createdDate": "2021-08-31T18:11:41Z",
+    #         "status": "NEW",
+    #         "ipAddress": "91.121.138.65",
+    #         "updatedBy": {
+    #             "firstName": "John Delaroderie",
+    #             "id": 84004451,
+    #             "lastName": "TAM-LAB",
+    #             "username": "tamde_hd1"
+    #         }
+    #     }
+    # },
+
+    csv_columns = [
+        'id', 'ipAddress', 'fqdn', 'port', 'operatingSystem', 'source', 'status',
+        'createdDate', 'updatedDate', 'updatedBy',
+        'BATCH_DATE', 'BATCH_NUMBER', 'Row_Last_Updated'
+    ]
+    return csv_columns
+
+
+def was_catalog_csv_column_types():
+
+    csv_columns = {
+        'id': 'INTEGER'
+    }
+    return csv_columns
+
 
 def status_table_csv_columns():  # Return list of csv columns
 
     csv_columns = [
-        'STATUS_NAME', 'STATUS_DETAIL', 'STATUS_COUNT'
+        'STATUS_NAME', 'STATUS_DETAIL', 'LAST_BATCH_PROCESSED', 'Row_Last_Updated'
     ]
     return csv_columns
 
@@ -1456,7 +1797,7 @@ def status_table_csv_columns():  # Return list of csv columns
 def status_table_csv_column_types():
 
     csv_columns = {
-        'STATUS_COUNT': 'INTEGER'
+        'LAST_BATCH_PROCESSED': 'TEXT'
     }
     return csv_columns
 
