@@ -104,7 +104,8 @@ def update_cred(new_cred):
         local_date = etld_lib_datetime.get_local_date()
         cred_template_string = re.sub('\$DATE', local_date, cred_template_string)
         cred_template_string = re.sub('username: initialuser', new_username, cred_template_string)
-        cred_template_string = re.sub('password: initialpassword', new_password, cred_template_string)
+        #cred_template_string = re.sub('password: initialpassword', new_password, cred_template_string)
+        cred_template_string = cred_template_string.replace('password: initialpassword', new_password)
         cred_template_string = re.sub('api_fqdn_server: qualysapi.qualys.com', new_api_fqdn_server,
                                       cred_template_string)
         cred_template_string = re.sub('gateway_fqdn_server: gateway.qg1.apps.qualys.com', new_gateway_fqdn_server,
@@ -164,7 +165,10 @@ def get_filesystem_cred():
             authorization = 'Basic ' + \
                             base64.b64encode(
                                 f"{cred.get('username')}:{cred.get('password')}".encode('utf-8')).decode('utf-8')
-            username, password = base64.b64decode(authorization.replace("Basic ", "")).decode('utf-8').split(":")
+            authorization_base64 = authorization.replace('Basic ', "")
+            authorization_decoded_bytearray = base64.b64decode(authorization_base64)
+            authorization_decoded_string = str(authorization_decoded_bytearray, 'UTF-8')
+            username, password = str(authorization_decoded_string).split(':', 1)
             cred_file_mode = stat.filemode(os.stat(cred_file).st_mode)
             etld_lib_functions.logger.info(f"Found Credentials, Ensure perms are correct for your company. "
                                            f"username: {username}, api_fqdn_server:  {api_fqdn_server}, "
@@ -221,140 +225,44 @@ def get_cred(cred_dict={}):
         cred['bearer'] = cred_dict['bearer']
     else:
         cred['bearer'] = ""
+    if 'timer_obj' in cred_dict.keys():
+        # setup for bearer token timer get_bearer_stored_in_env
+        cred['timer_obj'] = cred_dict['timer_obj']
+    else:
+        cred['timer_obj'] = None
 
     return {'api_fqdn_server': cred['api_fqdn_server'],
             'gateway_fqdn_server': cred['gateway_fqdn_server'],
             'authorization': cred['authorization'],
             'username': cred['username'],
             'password': cred['password'],
-            'bearer': cred['bearer']}
+            'bearer': cred['bearer'],
+            'timer_obj': cred['timer_obj']}
 
 
-def get_cookie(update_cookie=True):
-    one_hour = (60*60)  # (Sec * Min)
-    try:
-        if update_cookie is True:
-            qualys_logout()
-            qualys_login()
-        if Path(cookie_file).is_file():
-            age_of_file = etld_lib_datetime.get_seconds_since_last_file_modification(Path(cookie_file))
-            if age_of_file > one_hour:
-                qualys_logout()
-                qualys_login()
-        else:
-            qualys_login()
+def get_bearer_stored_in_env(update_bearer=True, cred=None):
+    if cred is None:
+        cred = {}
 
-        with open(cookie_file, 'r', encoding='utf-8') as f:
-            cookie = f.read().replace('\n', '').replace('\r', '')
-        cookie_file_mode = stat.filemode(os.stat(cookie_file).st_mode)
-        etld_lib_functions.logger.info(f"Cookie Set, Ensure perms are correct for your company. "
-                                       f"permissions: {cookie_file_mode}, cookie file: {str(cookie_file)} ")
-    except Exception as e:
-        etld_lib_functions.logger.error(f"Error in File: {__file__} Line: {etld_lib_functions.lineno()}")
-        etld_lib_functions.logger.error(f"               Credentials Dir:  {cred_dir}")
-        etld_lib_functions.logger.error(f"              Credentials File:  {cred_file}")
-        etld_lib_functions.logger.error(f"Exception: {e}")
-        exit(1)
-
-    return cookie
-
-
-# def get_bearer(update_bearer=True):
-#     one_hour = (60*60)  # (Sec * Min)
-#     try:
-#         if update_bearer is True:
-#             qualys_gateway_login()
-#         if Path(bearer_file).is_file():
-#             age_of_file = etld_lib_datetime.get_seconds_since_last_file_modification(Path(bearer_file))
-#             if age_of_file > one_hour:
-#                 qualys_gateway_login()
-#         else:
-#             qualys_gateway_login()
-#
-#         with open(bearer_file, 'r', encoding='utf-8') as f:
-#             bearer = f.read().replace('\n', '').replace('\r', '')
-#         bearer_file_mode = stat.filemode(os.stat(bearer_file).st_mode)
-#         etld_lib_functions.logger.info(f"Bearer Set, Ensure perms are correct for your company. "
-#                                        f"permissions: {bearer_file_mode}, bearer file: {str(bearer_file)} ")
-#     except Exception as e:
-#         etld_lib_functions.logger.error(f"               Credentials Dir:  {cred_dir}")
-#         etld_lib_functions.logger.error(f"              Credentials File:  {cred_file}")
-#         etld_lib_functions.logger.error(f"Exception: {e}")
-#         exit(1)
-#
-#     return bearer
-
-
-# def qualys_gateway_login_test():
-#     two_hours = (60*60)*2  # (Sec * Min) * 2 for two hours in seconds
-#     if Path(bearer_file).is_file():
-#         age_of_file = etld_lib_datetime.get_seconds_since_last_file_modification(Path(bearer_file))
-#         if age_of_file > two_hours:
-#             qualys_gateway_login()
-#     else:
-#         qualys_gateway_login()
-
-
-# def qualys_gateway_login():
-#     global login_gateway_failed
-#     global http_return_code
-#
-#     login_gateway_failed = True
-#     cred_dict = get_cred()
-#     if cred_dict['gateway_fqdn_server'] is None:
-#         etld_lib_functions.logger.error(f"Please add gateway_fqdn_server to your credentials file.")
-#         exit(1)
-#
-#     # Login to Qualys, return bearer token.
-#     url = f"https://{cred_dict['gateway_fqdn_server']}/auth"  # Qualys Endpoint
-#     payload = {'token': 'true', 'password': cred_dict['password'], 'username': cred_dict['username'],
-#                'permissions': 'true'}
-#     payload = urlencode(payload, quote_via=quote_plus)
-#
-#     headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
-#                'Content-Type': 'application/x-www-form-urlencoded',
-#                'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
-#
-#     try:
-#         if Path(bearer_file).is_file():
-#             Path(bearer_file).unlink()
-#         response = requests.request("POST", url, headers=headers, data=payload,
-#                                     verify=etld_lib_config.requests_module_tls_verify_status)
-#         http_return_code = response.status_code
-#         if response.status_code == 201:
-#             bearer = f"Bearer {response.text}"
-#             with open(bearer_file, 'w', encoding='utf-8') as bearerfile:
-#                 bearerfile.write(bearer)
-#             oschmod.set_mode(str(bearer_file), "u+rw,u-x,go-rwx")
-#             etld_lib_functions.logger.info(f"LOGIN - Qualys Gateway Login Success with user: {cred_dict['username']}")
-#             bearer_file_mode = stat.filemode(os.stat(bearer_file).st_mode)
-#             etld_lib_functions.logger.info(f"Bearer Set, Ensure perms are correct for your company. "
-#                                            f"permissions: {bearer_file_mode}, bearer file: {str(bearer_file)} ")
-#             login_gateway_failed = False
-#         else:
-#             etld_lib_functions.logger.error(f"Fail - Qualys Gateway Login Failed with user: {cred_dict['username']}")
-#             etld_lib_functions.logger.error(f"       HTTP {response.status_code}")
-#             etld_lib_functions.logger.error(f"       Verify Qualys username, password and "
-#                                             f"gateway_fqdn_server in Credentials File")
-#             etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
-#             etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
-#             etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
-#             etld_lib_functions.logger.error(f"         gateway_fqdn_server:  {cred_dict['gateway_fqdn_server']}")
-#             exit(1)
-#     except requests.exceptions.RequestException as e:
-#         etld_lib_functions.logger.error(f"Fail - Qualys Gateway Login Failed with user: {cred_dict['username']}")
-#         etld_lib_functions.logger.error(f"       Verify Qualys username, password and "
-#                                         f"api_fqdn_server, and gateway_fqdn_server in Credentials File")
-#         etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
-#         etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
-#         etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
-#         etld_lib_functions.logger.error(f"         gateway_fqdn_server:  {cred_dict['gateway_fqdn_server']}")
-#         etld_lib_functions.logger.error(f"Exception: {e}")
-#         exit(1)
-
-
-def get_bearer_stored_in_env(update_bearer=True, cred={}):
     max_age_of_bearer_token = 300
+
+    timer_obj_update_bearer = False
+    if 'timer_obj' in cred.keys() and cred['timer_obj'] is not None:
+        timer_obj: etld_lib_functions.TimerObj = cred['timer_obj']
+        time_elapsed = timer_obj.get_time_elapsed()
+        time_elapsed_flag = timer_obj.if_time_elapsed_past_max_time(max_time=max_age_of_bearer_token)
+        if time_elapsed_flag:
+            timer_obj_update_bearer = True
+            timer_obj.reset_timer()
+            etld_lib_functions.logger.info(f"time elapsed: {time_elapsed:,.0f} seconds")
+        else:
+            timer_obj_update_bearer = False
+        cred['timer_obj'] = timer_obj
+    else:
+        cred['timer_obj'] = etld_lib_functions.TimerObj()
+
+    # TODO move away from file to timerobj for bearer token update.
+
     try:
         if update_bearer:
             cred = qualys_gateway_login_store_in_env(cred_dict=cred)
@@ -364,7 +272,10 @@ def get_bearer_stored_in_env(update_bearer=True, cred={}):
             age_of_file = etld_lib_datetime.get_seconds_since_last_file_modification(Path(bearer_file))
             if age_of_file > max_age_of_bearer_token:
                 cred = qualys_gateway_login_store_in_env(cred_dict=cred)
-                etld_lib_functions.logger.info(f"Updated Bearer Token Successfully. age_of_file={age_of_file}, max_age={max_age_of_bearer_token}")
+                etld_lib_functions.logger.info(f"Updated Bearer Token Successfully. "
+                                               f"age_of_file={age_of_file}, max_age={max_age_of_bearer_token}")
+        elif timer_obj_update_bearer:
+            cred = qualys_gateway_login_store_in_env(cred_dict=cred)
         else:
             cred = qualys_gateway_login_store_in_env(cred_dict=cred)
 
@@ -419,8 +330,10 @@ def qualys_gateway_login_store_in_env(cred_dict={}):
     headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
                'Content-Type': 'application/x-www-form-urlencoded',
                'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
-    max_retries = 5
-    sleep_time = 10
+    max_retries = 15
+    sleep_time_requests_module_failed = 300
+    sleep_time_503 = 10
+    retry_number = 0
     for retry_number in range(max_retries):
         try:
             response = requests.request("POST", url, headers=headers, data=payload,
@@ -432,7 +345,7 @@ def qualys_gateway_login_store_in_env(cred_dict={}):
                     response=response,
                     cred_dict=cred_dict,
                     log_level=etld_lib_functions.logger.warning)
-                time.sleep(sleep_time)
+                time.sleep(sleep_time_503)
             elif response.status_code == 201:
                 cred_dict = update_cred_dict(response, cred_dict)
                 login_gateway_failed = False
@@ -446,14 +359,15 @@ def qualys_gateway_login_store_in_env(cred_dict={}):
                 exit(1)
         except requests.exceptions.RequestException as e:
             qualys_gateway_login_failed_message(
-                message=f"Requests Module Failed ",
+                message=f"Requests Module Failed {e}",
                 response="",
                 cred_dict=cred_dict,
-                log_level=etld_lib_functions.logger.error)
-            etld_lib_functions.logger.error(f"Exception: {e}")
-            exit(2)
+                log_level=etld_lib_functions.logger.warning)
+            etld_lib_functions.logger.warning(f"Exception: {e}")
+            time.sleep(sleep_time_requests_module_failed)
     else:
-        qualys_gateway_login_failed_message(response="",
+        qualys_gateway_login_failed_message(response="ALL RETRIES FAILED",
+                                            message=f"Retry number {retry_number} of {max_retries}",
                                             cred_dict=cred_dict,
                                             log_level=etld_lib_functions.logger.error)
         exit(3)
@@ -461,75 +375,84 @@ def qualys_gateway_login_store_in_env(cred_dict={}):
     return cred_dict
 
 
-def qualys_login():
-    global use_cookie
-    global login_failed
-    global http_return_code
-    login_failed = True
-    cred_dict = get_cred()
-    url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
-    payload = {'action': 'login', 'username': cred_dict['username'], 'password': cred_dict['password']}
-    payload = urlencode(payload, quote_via=quote_plus)
+def test_qualys_login_logout_basic_auth() -> dict:
 
-    headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
-               'Content-Type': 'application/x-www-form-urlencoded',
-               'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
-
-    try:
-        response = requests.request("POST", url, headers=headers, data=payload,
-                                    verify=etld_lib_config.requests_module_tls_verify_status)
-        http_return_code = response.status_code
-        if response.status_code == 200:
-            # cookie_dict = response.cookies.get_dict()
-            # cookie = f"DWRSESSIONID={cookie_dict['DWRSESSIONID']}; QualysSession={cookie_dict['QualysSession']}"
-            with open(cookie_file, 'w', encoding='utf-8') as cookiefile:
-                # cookiefile.write(cookie)
-                cookiefile.write("DUMMY")
-            oschmod.set_mode(str(cookie_file), "u+rw,u-x,go-rwx")
-            etld_lib_functions.logger.info(f"LOGIN - Qualys Login Success with user: {cred_dict['username']}")
-            login_failed = False
-        else:
-            etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user: {cred_dict['username']}")
-            etld_lib_functions.logger.error(f"       HTTP {response.status_code}")
-            etld_lib_functions.logger.error(f"       Verify Qualys username, password and "
-                                            f"api_fqdn_server in Credentials File")
-            etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
-            etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
-            etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
-            exit(1)
-        use_cookie = False
-    except requests.exceptions.RequestException as e:
-        etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user")
-        etld_lib_functions.logger.error(
-            f"       Verify Qualys username, password and api_fqdn_server in Credentials File")
-        etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
-        etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
-        etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
-        etld_lib_functions.logger.error(f"Exception: {e}")
-        exit(1)
+    def qualys_logout_test(login_status=None):
+        if login_status is not None and login_status['cookie'] != "":
+            cred_dict = get_cred()
+            url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
+            payload = {'action': 'logout'}
+            headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
+                       'Content-Type': 'application/x-www-form-urlencoded', 'Cookie': login_status['cookie'],
+                       'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
+            try:
+                response = requests.request("POST", url, headers=headers, data=payload,
+                                            verify=etld_lib_config.requests_module_tls_verify_status)
+                if response.status_code == 200:
+                    etld_lib_functions.logger.info(f"LOGOUT - Qualys Logout Success with user: {cred_dict['username']}")
+                else:
+                    etld_lib_functions.logger.warning(f"LOGOUT FAILED - probably stale cookie, continue with warning")
+            except Exception as e:
+                etld_lib_functions.logger.warning(f"LOGOUT FAILED, probably connectivity issue, continue with warning")
+                etld_lib_functions.logger.warning(f"Exception: {e}")
+                pass
 
 
-def qualys_logout():
-    global use_cookie
-    if use_cookie:
+    def qualys_login_test() -> dict:
+        login_status = {}
+        login_status['login_failed'] = True
+        login_status['cookie'] = ""
+
         cred_dict = get_cred()
         url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
-        payload = {'action': 'logout'}
+        payload = {'action': 'login', 'username': cred_dict['username'], 'password': cred_dict['password']}
+        payload = urlencode(payload, quote_via=quote_plus)
+
         headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
-                   'Content-Type': 'application/x-www-form-urlencoded', 'Cookie': get_cookie(update_cookie=False),
+                   'Content-Type': 'application/x-www-form-urlencoded',
                    'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
+
         try:
             response = requests.request("POST", url, headers=headers, data=payload,
                                         verify=etld_lib_config.requests_module_tls_verify_status)
+            login_status['response_status_code'] = response.status_code
             if response.status_code == 200:
-                etld_lib_functions.logger.info(f"LOGOUT - Qualys Logout Success with user: {cred_dict['username']}")
-            else:
-                etld_lib_functions.logger.warning(f"LOGOUT FAILED - probably stale cookie, continue with warning")
-        except Exception as e:
-            etld_lib_functions.logger.warning(f"LOGOUT FAILED, probably connectivity issue, continue with warning")
-            etld_lib_functions.logger.warning(f"Exception: {e}")
+                cookie_dict = response.cookies.get_dict()
+                cookie = f"DWRSESSIONID={cookie_dict['DWRSESSIONID']}; QualysSession={cookie_dict['QualysSession']}"
+                cookie = cookie.replace('\n', '').replace('\r', '')
 
-    use_cookie = False
+                etld_lib_functions.logger.info(f"LOGIN - Qualys Login Success with user: {cred_dict['username']}")
+
+                login_status['cookie'] = cookie
+                login_status['login_failed'] = False
+            else:
+                etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user: {cred_dict['username']}")
+                etld_lib_functions.logger.error(f"       HTTP {response.status_code}")
+                etld_lib_functions.logger.error(f"       Verify Qualys username, password and "
+                                                f"api_fqdn_server in Credentials File")
+                etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
+                etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
+                etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
+                exit(1)
+        except requests.exceptions.RequestException as e:
+            etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user")
+            etld_lib_functions.logger.error(
+                f"       Verify Qualys username, password and api_fqdn_server in Credentials File")
+            etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
+            etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
+            etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
+            etld_lib_functions.logger.error(f"Exception: {e}")
+            exit(1)
+
+        return login_status
+
+    login_status = qualys_login_test()
+    if login_status['login_failed']:
+        pass
+    else:
+        qualys_logout_test(login_status)
+
+    return login_status
 
 
 def main():
@@ -543,24 +466,10 @@ def main():
     bearer_file = Path(cred_dir, ".etld_bearer")  # Bearer File
     cred_file = Path(cred_dir, ".etld_cred.yaml")  # YAML Format Qualys Credentials
     use_cookie = False  # Set to true by qualys_login()
-    # Override if running from
-
-
-def test_basic_auth():
-    qualys_login()
-    time.sleep(2)
-    qualys_logout()
-    time.sleep(0.5)
-
-
-# def test_gateway_auth():
-#     qualys_gateway_login()
 
 
 if __name__ == '__main__':
     etld_lib_functions.main()
     etld_lib_config.main()
     main()
-    test_basic_auth()
-#    print(f"COOKIE: {get_cookie(update_cookie=False)}")
-#    print(f"BEARER: {get_bearer(update_bearer=False)}")
+    test_qualys_login_logout_basic_auth()
