@@ -4,6 +4,16 @@ import re
 from pathlib import Path
 
 
+def get_iso_datetime_string_minus_days(datetime_string="1970-01-02T00:00:00Z", days=1):
+    datetime_string = datetime_string.replace("T", " ")
+    datetime_string = datetime_string.replace("Z", "")
+    format_date = "%Y-%m-%d %H:%M:%S"
+    datetime_fixed = datetime.strptime(datetime_string, format_date)
+    d = datetime_fixed - timedelta(days=days)
+    td = f"{d.year}-{d.month:02d}-{d.day:02d}T00:00:00Z"
+    return td
+
+
 def get_utc_date_minus_days(days=1):
     d = datetime.utcnow() - timedelta(days=days)
     td = f"{d.year}-{d.month:02d}-{d.day:02d}T00:00:00Z"
@@ -46,6 +56,12 @@ def get_utc_datetime_qualys_format():
     return td
 
 
+def get_utc_datetime_sqlite_database_format():
+    d = datetime.utcnow()
+    td = f"{d.year}-{d.month:02d}-{d.day:02d} {d.hour:02d}:{d.minute:02d}:{d.second:02d}"
+    return td
+
+
 def get_local_date():
     d = datetime.now()
     td = f"{d.year}-{d.month:02d}-{d.day:02d} {d.hour:02d}:{d.minute:02d}:{d.second:02d}"
@@ -54,7 +70,7 @@ def get_local_date():
 
 def is_valid_qualys_datetime_format(qualys_datetime_format=None):
     # Light validation for YYYY-MM-DDThh:mm:ssZ format.
-    p_match = re.fullmatch(r"(^(19[7-9][0-9]|202[0-9])-"
+    p_match = re.fullmatch(r"(^(19[7-9][0-9]|200[0-9]|201[0-9]|202[0-9])-"
                            r"(0[1-9]|1[0-2])-"
                            r"(0[1-9]|1[0-9]|2[0-9]|3[0-1])T"
                            r"(0[0-9]|1[0-9]|2[0-3]):"
