@@ -112,7 +112,8 @@ def get_batch_number_from_filename(file_name):
 
 
 def get_batch_name_from_filename(file_name):
-    batch_name = re.sub("(^.*)(batch_[0-9]+)(\..*$)", "\g<2>", Path(file_name).name)
+    # batch_name = re.sub("(^.*)(batch_[0-9]+)(\..*$)", "\g<2>", Path(file_name).name)
+    batch_name = re.sub("(^.*)(batch_[a-z_0-9]+)(.*)", "\g<2>", Path(file_name).name)
     return batch_name
 
 
@@ -230,7 +231,19 @@ def extract_validation(validation_type='xml', output_file_to_validate=None, comp
         raise_exception_flag = False
         raise_exception_message_00 = f"JSON validation error file:{output_file_to_validate}"
         raise_exception_message_01 = "Undefined"
-        if str(output_file_to_validate).__contains__("asset_inventory_utc_run_datetime"):
+        if re.match("^.*was_.*_utc_.*$", str(output_file_to_validate)):
+            with compression_method(output_file_to_validate, "rt", encoding='utf-8') as read_file:
+                test_json_dict = json.load(read_file)
+                if 'ServiceResponse' in test_json_dict and 'responseCode' in test_json_dict['ServiceResponse']:
+                    if str(test_json_dict['ServiceResponse']['responseCode']).upper() == 'SUCCESS':
+                        pass
+                    else:
+                        raise_exception_message_01 = "responseCode not equal to SUCCESS"
+                        raise Exception(f"{raise_exception_message_01}, {raise_exception_message_00}")
+                else:
+                    raise_exception_message_01 = "ServiceResponse or responseCode not in data."
+                    raise Exception(f"{raise_exception_message_01}, {raise_exception_message_00}")
+        elif str(output_file_to_validate).__contains__("asset_inventory_utc_run_datetime"):
             with compression_method(output_file_to_validate, "rt", encoding='utf-8') as read_file:
                 asset_inventory_dict = json.load(read_file)
                 if 'responseCode' in asset_inventory_dict.keys():
@@ -268,10 +281,11 @@ def extract_qualys(
         output_file=None,
         cred_dict=None,
         qualys_headers_multiprocessing_dict=None,
-        multi_proc_batch_number=None,
+        batch_number_formatted=None,
         extract_validation_type='xml',
         requests_module_tls_verify_status=True,
-        compression_method=gzip.open
+        compression_method=gzip.open,
+        request_method="POST"
 ):
 
     # TODO For 401 Unauthorized with expired token, retry.  This is an extreme edge case.
@@ -280,22 +294,14 @@ def extract_qualys(
     for _ in range(try_extract_max_count):
         try:
             headers['User-Agent'] = f"qualysetl_v{qualys_etl.__version__}"
-            with requests.request("POST", url, stream=True, headers=headers, data=payload,
+            with requests.request(request_method, url, stream=True, headers=headers, data=payload,
                                   timeout=http_conn_timeout, verify=requests_module_tls_verify_status) as r:
                 qualys_headers = get_qualys_headers(r)
-                if multi_proc_batch_number is None:
+                if batch_number_formatted is None:
                     qualys_headers_multiprocessing_dict['batch_000001'] = get_qualys_headers(r)
                 else:
-                    qualys_headers_multiprocessing_dict[multi_proc_batch_number] = get_qualys_headers(r)
-                    etld_lib_functions.logger.info(f"Testing: {multi_proc_batch_number}")
-
-                    # if debug_error_code is True and multi_proc_batch_number == 'batch_000003':
-                    #     debug_error_code = False
-                    #     r.status_code = 401
-                    #
-                    # if debug_error_code is True and multi_proc_batch_number == 'batch_000005':
-                    #     debug_error_code = False
-                    #     r.status_code = 401
+                    qualys_headers_multiprocessing_dict[batch_number_formatted] = get_qualys_headers(r)
+                    etld_lib_functions.logger.info(f"Testing: {batch_number_formatted}")
 
                 etld_lib_functions.logger.info(f"Qualys Headers: {qualys_headers}")
                 if r.status_code == 200:
@@ -328,6 +334,11 @@ def extract_qualys(
                                                           f"retrying cred gateway for:{url}")
                         cred_dict = etld_lib_credentials.get_bearer_stored_in_env(update_bearer=True, cred=cred_dict)
                         raise Exception(f"HTTP Status is: {r.status_code}, message: {message}")
+                    else:
+                        message = get_http_error_code_message_v2_api(str(r.status_code))
+                        etld_lib_functions.logger.error(f"HTTP USER: {cred_dict['username']} url: {url}")
+                        etld_lib_functions.logger.error(f"HTTP {r.status_code}, exiting. message={message}")
+                        exit(1)
                 else:
                     message = get_http_error_code_message_v2_api(str(r.status_code))
                     etld_lib_functions.logger.error(f"HTTP USER: {cred_dict['username']} url: {url}")
