@@ -14,6 +14,7 @@ from qualys_etl.etld_knowledgebase import knowledgebase_02_workflow_manager
 from qualys_etl.etld_host_list import host_list_02_workflow_manager
 from qualys_etl.etld_host_list_detection import host_list_detection_02_workflow_manager
 from qualys_etl.etld_asset_inventory import asset_inventory_02_workflow_manager
+from qualys_etl.etld_was import was_02_workflow_manager
 from qualys_etl.etld_was import was_04_extract_from_qualys
 
 
@@ -45,13 +46,13 @@ def validate_json_file(open_file_method=open, file_path: Path = Path()):
                             f"Exception={e}"
                             )
         # TODO hasMoreRecords check for key and retry if key is missing.
-        # CSAM type is ...
-        # WAS type is
+        # CSAM etl_workflow_validation_type is ...
+        # WAS etl_workflow_validation_type is
         etld_lib_functions.logger.info(f"PASS: JSON VALIDATED, "
                                        f"validation_test_type={validation_type}, "
                                        f"directory={Path(file_path).parent.name}, "
                                        f"batch file={etld_lib_extract_transform_load.get_batch_name_from_filename(file_path)}, "
-                                       f"tail file={last_characters}")
+                                       f"tail file={last_characters.replace('|', '')}")
 
 
 def validate_extract_directories_json_files_downloaded():
@@ -60,6 +61,7 @@ def validate_extract_directories_json_files_downloaded():
     json_dir_list.append(Path(etld_lib_config.host_list_detection_extract_dir))
     json_dir_list.append(Path(etld_lib_config.kb_extract_dir))
     json_dir_list.append(Path(etld_lib_config.asset_inventory_extract_dir))
+    json_dir_list.append(Path(etld_lib_config.was_extract_dir))
     json_dir_search_glob = '*.json.gz'
 
     for json_dir in json_dir_list:
@@ -130,7 +132,7 @@ def validate_xml_is_closed_properly(test_by='xml_characters',
         else:
             xml_lines = tail_file_by_lines(open_file_method, file_path, number_of_lines)
     except Exception as e:
-        xml_line = ''.join(xml_lines.replace('\n', ''))
+        xml_line = ''.join(xml_lines.replace('\n', '').replace('|', ''))
         raise Exception(f"WARNING: CANNOT READ, CREATION MAY BE IN PROGRESS "
                         f"test_by={test_by} "
                         f"directory={directory_name} "
@@ -142,7 +144,7 @@ def validate_xml_is_closed_properly(test_by='xml_characters',
     end_of_response = '</RESPONSE>'
     found_incident = False
     incident_text = 'incident signature'
-    xml_line = ''.join(xml_lines.replace('\n', ''))
+    xml_line = ''.join(xml_lines.replace('\n', '').replace('|', ''))
     if end_of_response in xml_line:
         found_end_of_response = True
     if incident_text in str(xml_line).lower():
@@ -211,6 +213,7 @@ def test_host_list(test_name='host_list'):
     etld_lib_config.host_list_vm_processed_after = etld_lib_datetime.get_utc_date_minus_days(180)
     etld_lib_config.host_list_test_system_flag = True
     etld_lib_config.host_list_test_number_of_files_to_extract = 3
+    etld_lib_config.host_list_distribution_csv_flag = True
     etld_lib_config.host_list_payload_option = {'truncation_limit': '25'}
     etld_lib_functions.logger.info(
         f"Starting test with: "
@@ -232,6 +235,7 @@ def test_host_list_detection(test_name='host_list_detection'):
     etld_lib_config.host_list_vm_processed_after = etld_lib_datetime.get_utc_date_minus_days(180)
     etld_lib_config.host_list_test_system_flag = True
     etld_lib_config.host_list_test_number_of_files_to_extract = 3
+    etld_lib_config.host_list_detection_distribution_csv_flag = True
     etld_lib_config.host_list_payload_option = {'truncation_limit': '25'}
     etld_lib_functions.logger.info(
         f"Starting test with: "
@@ -252,6 +256,7 @@ def test_asset_inventory(test_name='asset_inventory'):
     etld_lib_config.asset_inventory_asset_last_updated = etld_lib_datetime.get_utc_date_minus_days(180)
     etld_lib_config.asset_inventory_test_system_flag = True
     etld_lib_config.asset_inventory_test_number_of_files_to_extract = 3
+    etld_lib_config.asset_inventory_distribution_csv_flag = True
     etld_lib_functions.logger.info(
         f"Starting test with: "
         f"asset_inventory_asset_last_updated={etld_lib_config.asset_inventory_asset_last_updated}, "
@@ -270,11 +275,35 @@ def test_asset_inventory(test_name='asset_inventory'):
         f"{etld_lib_config.asset_inventory_test_number_of_files_to_extract}")
 
 
+def test_was(test_name='was'):
+    etld_lib_config.was_webapp_last_scan_date = etld_lib_datetime.get_utc_date_minus_days(365)
+    etld_lib_config.was_test_system_flag = True
+    etld_lib_config.was_test_number_of_files_to_extract = 3
+    etld_lib_config.was_distribution_csv_flag = True
+    etld_lib_functions.logger.info(
+        f"Starting test with: "
+        f"was_webapp_last_scan_date={etld_lib_config.was_webapp_last_scan_date}, "
+        f"was_test_number_of_files_to_extract="
+        f"{etld_lib_config.was_test_number_of_files_to_extract}")
+    try:
+        was_02_workflow_manager.main()
+    except Exception as e:
+        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
+        raise Exception
+
+    etld_lib_functions.logger.info(
+        f"Ending test with: "
+        f"was_webapp_last_scan_date={etld_lib_config.was_webapp_last_scan_date}, "
+        f"was_test_number_of_files_to_extract="
+        f"{etld_lib_config.was_test_number_of_files_to_extract}")
+
+
 def test_was_count():
     etld_lib_config.remove_old_files(
         dir_path=etld_lib_config.was_extract_dir,
         dir_search_glob=etld_lib_config.was_extract_dir_file_search_blob_webapp_count
-                                     )
+    )
+
     file_info_dict = \
         etld_lib_config.prepare_extract_batch_file_name(
             next_batch_number_str='batch_000001',
@@ -302,6 +331,7 @@ def test_was_count():
 
 
 def main(test_function_to_run='All'):
+    etld_lib_config.test_system_do_not_test_intermediary_extracts_flag = True
     return_code_dict = {}
     return_code = 0
     try:
@@ -332,6 +362,13 @@ def main(test_function_to_run='All'):
         return_code_dict['test_asset_inventory'] = 1
         return_code_dict['test_asset_inventory_exception'] = f"{e}"
         return_code = 1
+    try:
+        if test_function_to_run == 'All' or test_function_to_run == 'test_was':
+            test_was()
+    except Exception as e:
+        return_code_dict['test_was'] = 1
+        return_code_dict['test_was_exception'] = f"{e}"
+        return_code = 1
     try:
         if test_function_to_run == 'All' or test_function_to_run == 'validate_extract_directories_json_files_downloaded':
             validate_extract_directories_json_files_downloaded()
@@ -346,13 +383,13 @@ def main(test_function_to_run='All'):
         return_code_dict['test_extract_directories_xml_files_downloaded'] = 1
         return_code_dict['test_extract_directories_xml_files_downloaded_exception'] = f"{e}"
         return_code = 1
-    try:
-        if test_function_to_run == 'test_was_count':
-            test_was_count()
-    except Exception as e:
-        return_code_dict['test_was_count'] = 1
-        return_code_dict['test_was_count_exception'] = f"{e}"
-        return_code = 1
+    # try:
+    #     if test_function_to_run == 'test_was_count':
+    #         test_was_count()
+    # except Exception as e:
+    #     return_code_dict['test_was_count'] = 1
+    #     return_code_dict['test_was_count_exception'] = f"{e}"
+    #     return_code = 1
 
     for return_code_key in return_code_dict.keys():
         etld_lib_functions.logger(f"TEST FAILED: {return_code_key}={return_code_dict[return_code_key]}")
