@@ -320,6 +320,24 @@ def start_etl_test_system():
     print(f"End      etl_test_system.  For progress see your {etld_lib_config.test_system_log_file}")
 
 
+def start_etl_validate(etl_option=""):
+    from qualys_etl.etld_lib import etld_lib_log_validation
+    try:
+        if etl_option == 'validate_etl_all':
+            etld_lib_log_validation.main_validate_all_logs_have_no_errors()
+        elif etld_lib_log_validation.main_validate_log_has_no_errors(etl_option):
+            pass
+        else:
+           exit(1)
+    except FileNotFoundError as fnf:
+        print(f"Log File Not Found: {fnf}")
+        print(f"{etl_option} validation log not found, please rerun after executing corresponding {etl_option.replace('validate_', '')}")
+        exit(1)
+    except Exception as e:
+        help_message(f"-e {etl_option} is not supported.")
+        exit(1)
+
+
 def validate_and_set_command_line_arguments_datetime_in_etld_lib_config():
     if command_line_arguments.datetime is None:
         pass
@@ -335,7 +353,9 @@ def validate_and_set_command_line_arguments_datetime_in_etld_lib_config():
 
 def if_user_selected_execute_an_etl_module():
     validate_and_set_command_line_arguments_datetime_in_etld_lib_config()
-    if command_line_arguments.execute_etl_module == 'etl_knowledgebase':
+    if str(command_line_arguments.execute_etl_module).startswith("validate_etl_"):
+        start_etl_validate(etl_option=command_line_arguments.execute_etl_module)
+    elif command_line_arguments.execute_etl_module == 'etl_knowledgebase':
         start_etl_knowledgebase()
     elif command_line_arguments.execute_etl_module == 'etl_host_list':
         start_etl_host_list()
@@ -352,14 +372,10 @@ def if_user_selected_execute_an_etl_module():
     elif command_line_arguments.execute_etl_module is None:
         pass
     else:
-        etld_lib_functions.logger.info(f"Invalid Option: {str(command_line_arguments.execute_etl_module)}, "
-                                       f"retry when ready.")
+        #etld_lib_functions.logger.info(f"Invalid Option: {str(command_line_arguments.execute_etl_module)}, "
+        #                               f"retry when ready.")
         print(f"\nInvalid Option: {str(command_line_arguments.execute_etl_module)}, retry when ready")
-        print(f"Options are: -e etl_knowledgebase ")
-        print(f"Options are: -e etl_host_list")
-        print(f"Options are: -e etl_host_list_detection")
-        print(f"Options are: -e etl_asset_inventory")
-        print(f"Options are: -e etl_test_system")
+        help_message("")
         exit(1)
 
 
@@ -371,11 +387,11 @@ def if_user_selected_test_qualys_basic_authentication_login(first_time=False):
         credentials = etld_lib_credentials.get_cred()
         print(f"Qualys Login Test for {credentials.get('username')} "
               f"at api_fqdn_server: {credentials.get('api_fqdn_server')}\n")
-        etld_lib_credentials.test_basic_auth()
-        if etld_lib_credentials.login_failed is not True:
-            print(f"Testing Qualys Login for {credentials.get('username')} "
+        login_status = etld_lib_credentials.test_qualys_login_logout_basic_auth()
+        if login_status['login_failed'] is not True:
+            print(f"Testing Qualys Login/Logout for {credentials.get('username')} "
                   f"Succeeded at {credentials.get('api_fqdn_server')}\n"
-                  f"    with HTTPS Return Code: {etld_lib_credentials.http_return_code}.")
+                  f"    with HTTPS Return Code: {login_status['response_status_code']}.")
         etld_lib_functions.main(log_level=logging.WARN, my_logger_prog_name='qetl_manage_user')
         return True
     else:
@@ -413,33 +429,60 @@ def if_user_selected_print_directory_listing_report():
 
 def help_message(notes):
     help_mess = f'''
-        
     {notes}
-        
-    usage: qetl_manage_user [-h] -u qetl_USER_HOME_DIR [-e execute etl module] [-c] [-t] [-d] [-r] [-l]
+    usage: qetl_manage_user [-h] [-u qetl_USER_HOME_DIR] [-e etl_[module] ] [-e validate_etl_[module] ] [-c] [-t] [-i] [-d] [-r] [-l]
     
     Command to Extract, Transform and Load Qualys Data into various forms ( CSV, JSON, SQLITE3 DATABASE )
     
     optional arguments:
       -h, --help                show this help message and exit
-      -u Home Directory Path, --qetl_user_home_dir Home directory Path
+      -u Home Directory Path,   --qetl_user_home_dir Home directory Path
                                    Example:
                                    - /opt/qetl/users/q_username
-      -e etl module name,     --execute_etl_module module name
-                                  Ex.  -e etl_knowledgebase or -e etl_host_list or -e etl_host_list_detection -e etl_asset_inventory
-                                  Ex.  -e etl_test_system ( for small system test of all ETL jobs )
-      -d YYMMDDThh:mm:ssZ,    --datetime        YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date. 
+      -e etl_[module],          --execute_etl_[module] execute etl of module name. valid options are:
+                                       -e etl_knowledgebase 
+                                       -e etl_host_list 
+                                       -e etl_host_list_detection
+                                       -e etl_asset_inventory
+                                       -e etl_was
+                                       -e etl_test_system ( for a small system test of all ETL Jobs )
+      -e validate_etl_[module], --validate_etl_[module] [test last run of etl_[module]].  valid options are:
+                                       -e validate_etl_knowledgebase
+                                       -e validate_etl_host_list 
+                                       -e validate_etl_host_list_detection
+                                       -e validate_etl_asset_inventory
+                                       -e validate_etl_was
+                                       -e validate_etl_test_system 
+      -d YYMMDDThh:mm:ssZ,      --datetime      YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date. 
                                                 Ex. 1970-01-01T00:00:00Z acts as flag to obtain all data.
-      -c, --credentials       update qualys api user credentials: qualys username, password or api_fqdn_server
-      -t, --test              test qualys credentials
-      -l, --logs              detailed logs sent to stdout for testing qualys credentials
-      -v, --version           Help and QualysETL version information.
-      -r, --report            brief report of the users directory structure.
+      -c, --credentials        update qualys api user credentials: qualys username, password or api_fqdn_server
+      -t, --test               test qualys credentials
+      -i, --initialize_user    For automation, create a /opt/qetl/users/[userhome] directory 
+                               without being prompted.
+      -l, --logs               detailed logs sent to stdout for testing qualys credentials
+      -v, --version            Help and QualysETL version information.
+      -r, --report             brief report of the users directory structure.
       -p, --prompt-credentials prompt user for credentials, also accepts stdin with credentials piped to program.
-      -m, --memory-credentials get credentials from environment: q_username, q_password, q_api_fqdn_server
+      -m, --memory-credentials get credentials from environment: 
+                               Example: q_username="your userid", q_password=your password, q_api_fqdn_server=api fqdn, q_gateway_fqdn_server=gateway api fqdn
       -s, --stdin-credentials  send credentials in json to stdin. 
                                Example:
                                {{"q_username": "your userid", "q_password": "your password", "q_api_fqdn_server": "api fqdn", "q_gateway_fqdn_server": "gateway api fqdn"}}
+      
+      etld_config_settings.yaml notes:
+         1. To Enable CSV Distribution, add the following keys to etld_config_settings.yaml and toggle on/off them via True or False
+              kb_distribution_csv_flag: True                    # populates qetl_home/data/knowledgebase_distribution_dir
+              host_list_distribution_csv_flag: True             # populates qetl_home/data/host_list_distribution_dir
+              host_list_detection_distribution_csv_flag: True   # populates qetl_home/data/host_list_detection_distribution_dir
+              asset_inventory_distribution_csv_flag: True       # populates qetl_home/data/asset_inventory_distribution_dir
+              was_distribution_csv_flag: True                   # populates qetl_home/data/was_distribution_dir
+              
+              These files are prepared for database load, tested with mysql.  No headers are present.  
+              Contact your Qualys TAM and schedule a call with David Gregory if you need assistance with this option.
+            
+         2. To enable TruRisk, ask your TAM to add TruRisk to your subscription, then update the following keys in your etld_config_settings.yaml
+            host_list_payload_option: {{'show_ars': '1', 'show_ars_factors': '1'}}
+            host_list_detection_payload_option: {{'show_qds': '1', 'show_qds_factors': '1'}}
                                
       QualysETL Version: {qualys_etl.__version__}
     '''
@@ -450,6 +493,11 @@ def test_command_line_arguments():
     global command_line_arguments
     # If no options check for -u.  If -u exists, help_message.  If -u does not exist, continue to new user prompts.
     # If -u is not set, help_message
+
+    if command_line_arguments.version:
+        help_message(notes="")
+        exit(0)
+
     if command_line_arguments.qetl_user_home_dir is None:
         help_message(f"Please enter -u [ your /opt/qetl/users/ user home directory path ]\n\n"
                      f"    Note: /opt/qetl/users/newuser is the root directory for your qetl userhome directory,\n" 
@@ -466,7 +514,7 @@ def test_command_line_arguments():
             test_qetl_user_home_dir.parent.parent.parent.name == 'opt':
         pass
     else:
-        help_message(f"Please check permissions on {test_qetl_user_home_dir.parent.parent},\n "
+        help_message(f"Please check path and permissions on {test_qetl_user_home_dir.parent.parent},\n "
                      f"   You don't appear to have authorization to write to that directory.\n")
         exit(1)
 
@@ -475,6 +523,7 @@ def test_command_line_arguments():
                 command_line_arguments.credentials is None and \
                 command_line_arguments.test is None and \
                 command_line_arguments.logs is None and \
+                command_line_arguments.initialize_user is None and \
                 command_line_arguments.report is None:
             help_message(f"Please select an option for qetl_user: {command_line_arguments.qetl_user_home_dir}")
             exit(1)
@@ -485,14 +534,18 @@ def test_command_line_arguments():
 def get_command_line_arguments(args=None):
     global command_line_arguments
     parser = argparse.ArgumentParser(description='Command to Extract, Transform and Load Qualys Data into various forms ( CSV, JSON, SQLITE3 DATABASE )')
-    parser.add_argument('-u', '--qetl_user_home_dir', default=None, help="Please enter -u option")
+    parser.add_argument('-u', '--qetl_user_home_dir', default=None, help="Please enter -u option with path to your "
+                                                                         "user directory.  Ex. /opt/qetl/users/[youruserdir]")
     parser.add_argument('-e', '--execute_etl_module', default=None,
-                        help='Execute etl_knowledgebase, etl_host_list, etl_host_list_detection, etl_asset_inventory')
+                        help='etl_knowledgebase, etl_host_list, etl_host_list_detection, etl_asset_inventory, etl_was, '
+                             'validate_etl_knowledgebase, validate_etl_host_list, validate_etl_host_list_detection, '
+                             'validate_etl_was, validate_etl_all, validate_etl_test_system')
     parser.add_argument('-d', '--datetime', default=None,
                         help='YYYY-MM-DDThh:mm:ssZ UTC. Get All Data On or After Date.  Ex. 1970-01-01T00:00:00Z acts as flag to obtain all data.')
     parser.add_argument('-c', '--credentials', default=None, action="store_true",
                         help='update qualys api user credentials stored on disk: qualys username, password or api_fqdn_server')
     parser.add_argument('-t', '--test', default=None, action="store_true", help='test qualys credentials')
+    parser.add_argument('-i', '--initialize_user', default=None, action="store_true", help='create qualys user directory')
     parser.add_argument('-l', '--logs', default=None, action="store_true", help='detailed logs sent to stdout for test qualys credentials')
     parser.add_argument('-p', '--prompt_credentials', default=None, action="store_true", help='prompt user for credentials')
     parser.add_argument('-s', '--stdin_credentials', default=None, action="store_true",
@@ -501,6 +554,12 @@ def get_command_line_arguments(args=None):
                         help='Get credentials from environment variables in memory: q_username, q_password, q_api_fqdn_server, and optionally add q_gateway_fqdn_server. Ex. export q_username=myuser')
     parser.add_argument('-v', '--version', default=None, action="store_true", help='Help and QualysETL Version')
     parser.add_argument('-r', '--report', default=None, action="store_true", help='Brief report of the users directory structure.')
+    # Print help menu from -v option.
+    for idx, arg in enumerate(args):
+        if '-h' in arg:
+            args[idx] = '-v'
+        if '--help' in arg:
+            args[idx] = '-v'
     command_line_arguments = parser.parse_args(args)
 
 
@@ -525,8 +584,14 @@ def setup_qualys_etl_user_home_environment_in_memory_and_in_etld_lib_config():
     else:
         # Potential New User, Query for confirmation
         time.sleep(1)
-        print(f"\nqetl_user_home_dir does not exist: {etld_lib_config.qetl_user_home_dir}")
-        response = input(f"Create new qetl_user_home_dir? {etld_lib_config.qetl_user_home_dir} ( yes or no ): ")
+        qetl_manage_user_init=""
+        if command_line_arguments.initialize_user is not None:
+            print(f"\nqetl_user_home_dir does not exist: {etld_lib_config.qetl_user_home_dir}")
+            response = 'qetl_manage_user_init'
+        else:
+            print(f"\nqetl_user_home_dir does not exist: {etld_lib_config.qetl_user_home_dir}")
+            response = input(f"Create new qetl_user_home_dir? {etld_lib_config.qetl_user_home_dir} ( yes or no ): ")
+
         if response == 'yes':
             if command_line_arguments.execute_etl_module is not None:
                 if command_line_arguments.logs is None:
@@ -542,12 +607,16 @@ def setup_qualys_etl_user_home_environment_in_memory_and_in_etld_lib_config():
             if response == 'yes':
                 print("")
                 if_user_selected_test_qualys_basic_authentication_login(first_time=True)
-            # response = input(f"\nWould you like to test login/logout of Qualys Gateway? ( yes or no ): ")
-            # if response == 'yes':
-            #     print("")
-            #     if_user_selected_test_qualys_gateway_token_login(first_time=True)
             print(f"\nThank you, exiting.\n")
             exit(0)
+        elif response == 'qetl_manage_user_init':
+            etld_lib_config.qetl_create_user_dirs_ok_flag = True
+            etld_lib_functions.main(log_level=logging.WARNING, my_logger_prog_name='qetl_manage_user_init')
+            etld_lib_config.main()
+            time.sleep(1)
+            print(f"\nqetl_user_home_dir created: {etld_lib_config.qetl_user_home_dir}")
+            print(f"Thank you, exiting.\n")
+            exit(0)
         else:
             print(f"\nThank you, exiting.\n")
             exit(1)
